"""
星表から星空を作る

データ取得元：Astro Commons; ヒッパルコス星表
http://astro.starfree.jp/commons/hip/
（一部抜粋し、有効桁数を3桁に修正）
"""

import csv
from io import StringIO, TextIOBase
from itertools import chain, islice

import bpy
import numpy as np


class Table:
    def __init__(self, data, names=None):
        if isinstance(data, dict):
            self.columns = data
            return
        if names is None:
            names = data[0]
            data = data[1:]
        self.columns = {}
        for name, column in zip(names, data.T):
            try:
                column = column.astype(int)
            except ValueError:
                try:
                    column = column.astype(float)
                except ValueError:
                    pass
            self.columns[name] = column

    def __getitem__(self, name):
        if isinstance(name, list):
            return Table({k: self.columns[k] for k in name})
        return self.columns[name]

    def __str__(self):
        rows = [",".join(self.columns)]
        for row in zip(*self.columns.values()):
            rows.append(",".join(map(str, row)))
        return "\n".join(rows)

    def __repr__(self):
        columns = self.columns
        n = len(next(iter(columns.values())))
        if len(columns) > 5:
            columns = chain(islice(columns, 4), ["..."])
        return f"<Table [{', '.join(columns)}] {n}rows>"

    @property
    def values(self):
        return np.array(list(self.columns.values())).T

    @staticmethod
    def read_csv(path, encoding=None, names=None):
        with path if isinstance(path, TextIOBase) else open(path, encoding=encoding) as fp:
            return Table(np.array([row for row in csv.reader(fp)]), names=names)


def make_material(obj):
    mat = bpy.data.materials.new(name="line")
    mat.use_nodes = True
    nd_pb = mat.node_tree.nodes["Principled BSDF"]
    nd_pb.inputs[26].default_value = 0.05, 0.05, 0.05, 1
    nd_pb.inputs[27].default_value = 1

    mat = bpy.data.materials.new(name="star")
    obj.active_material = mat
    mat.use_nodes = True
    nd_pb = mat.node_tree.nodes["Principled BSDF"]
    nd_a1 = mat.node_tree.nodes.new("ShaderNodeAttribute")
    nd_a1.attribute_name = "bv"
    nd_a1.attribute_type = "INSTANCER"
    nd_a2 = mat.node_tree.nodes.new("ShaderNodeAttribute")
    nd_a2.attribute_name = "lv"
    nd_a2.attribute_type = "INSTANCER"
    nd_cr = mat.node_tree.nodes.new("ShaderNodeValToRGB")
    nd_cr.color_ramp.color_mode = "HSV"
    nd_cr.color_ramp.hue_interpolation = "CCW"
    nd_cr.color_ramp.elements.new(0.95)
    nd_cr.color_ramp.elements[0].color = 0, 0.1, 1, 1
    nd_cr.color_ramp.elements[1].color = 1, 0, 0, 1
    nd_cr.color_ramp.elements[2].color = 0.01, 0.01, 0.01, 1
    nd_a1.location = -450, -180
    nd_a2.location = -450, 5
    nd_cr.location = -270, 5
    mat.node_tree.links.new(nd_a1.outputs[2], nd_cr.inputs[0])
    mat.node_tree.links.new(nd_cr.outputs[0], nd_pb.inputs[26])
    mat.node_tree.links.new(nd_a2.outputs[2], nd_pb.inputs[27])


def make_geometry_node(obj):
    modifier = obj.modifiers.new("GeometryNodes", "NODES")
    node_group = bpy.data.node_groups.new("Geometry Nodes", "GeometryNodeTree")
    modifier.node_group = node_group
    node_group.is_modifier = True
    node_group.interface.new_socket("Geometry", in_out="OUTPUT", socket_type="NodeSocketGeometry")
    node_group.interface.new_socket("Geometry", in_out="INPUT", socket_type="NodeSocketGeometry")
    nd_gi = node_group.nodes.new("NodeGroupInput")
    nd_ic = node_group.nodes.new("GeometryNodeMeshIcoSphere")
    nd_ic.inputs[0].default_value = 0.003
    nd_io = node_group.nodes.new("GeometryNodeInstanceOnPoints")
    nd_mc = node_group.nodes.new("GeometryNodeMeshToCurve")
    nd_cc = node_group.nodes.new("GeometryNodeCurvePrimitiveCircle")
    nd_cc.inputs[0].default_value = 6
    nd_cc.inputs[4].default_value = 0.001
    nd_cm = node_group.nodes.new("GeometryNodeCurveToMesh")
    nd_sm = node_group.nodes.new("GeometryNodeSetMaterial")
    nd_sm.inputs[2].default_value = bpy.data.materials["star"]
    nd_sm2 = node_group.nodes.new("GeometryNodeSetMaterial")
    nd_sm2.inputs[2].default_value = bpy.data.materials["line"]
    nd_jo = node_group.nodes.new("GeometryNodeJoinGeometry")
    nd_go = node_group.nodes.new("NodeGroupOutput")

    node_group.links.new(nd_gi.outputs[0], nd_io.inputs[0])
    node_group.links.new(nd_ic.outputs[0], nd_io.inputs[2])
    node_group.links.new(nd_gi.outputs[0], nd_mc.inputs[0])
    node_group.links.new(nd_mc.outputs[0], nd_cm.inputs[0])
    node_group.links.new(nd_cc.outputs[0], nd_cm.inputs[1])
    node_group.links.new(nd_io.outputs[0], nd_sm.inputs[0])
    node_group.links.new(nd_cm.outputs[0], nd_sm2.inputs[0])
    node_group.links.new(nd_sm.outputs[0], nd_jo.inputs[0])
    node_group.links.new(nd_sm2.outputs[0], nd_jo.inputs[0])
    node_group.links.new(nd_jo.outputs[0], nd_go.inputs[0])

    nds = [nd_gi, nd_ic, nd_io, nd_mc, nd_cc, nd_cm, nd_sm, nd_sm2, nd_jo, nd_go]
    locs = [[-315, 116], [-315, -18], [-151, 116], [8, 116], [8, -18]]
    locs += [[170, 116], [170, -18], [325, 116], [325, -18], [482, 116]]
    for nd, loc in zip(nds, locs):
        nd.location = loc


def make_object():
    df = Table.read_csv(StringIO(df_str))
    dfl = Table.read_csv(StringIO(dfl_str))
    verts = (df[["X", "Y", "Z"]].values / 10).tolist()
    edges = dfl[["ID1", "ID2"]].values.tolist()
    mesh = bpy.data.meshes.new(name="Star")
    mesh.from_pydata(verts, edges, [])
    obj = bpy.data.objects.new(mesh.name, mesh)
    bpy.context.scene.collection.objects.link(obj)
    mesh.attributes.new("lv", "FLOAT", "POINT")
    mesh.attributes["lv"].data.foreach_set("value", ((4 - df["V"]) ** 2 * 5).tolist())
    mn, mx = df["B"].min(), df["B"].max()
    mesh.attributes.new("bv", "FLOAT", "POINT")
    mesh.attributes["bv"].data.foreach_set("value", ((df["B"] - mn) / (mx - mn)).tolist())
    make_material(obj)
    make_geometry_node(obj)
    bpy.data.worlds["World"].node_tree.nodes["Background"].inputs[0].default_value = 0, 0, 0, 1
    set_bloom()


def set_bloom():
    scene = bpy.data.scenes["Scene"]
    scene.use_nodes = True
    nodes = scene.node_tree.nodes
    glare = nodes.new(type="CompositorNodeGlare")
    glare.glare_type = "BLOOM"
    glare.quality = "LOW"
    glare.location = 50, 200
    node1 = scene.node_tree.nodes["Render Layers"]
    node2 = scene.node_tree.nodes["Composite"]
    scene.node_tree.links.new(node1.outputs[0], glare.inputs[0])
    scene.node_tree.links.new(glare.outputs[0], node2.inputs[0])
    for area in bpy.data.screens["Layout"].areas:
        if area.ui_type == "VIEW_3D":
            area.spaces[0].shading.type = "RENDERED"
            area.spaces[0].shading.use_compositor = "ALWAYS"


df_str = """\
V,B,X,Y,Z
5.71,2.0,6.586,0.031,-7.525
4.78,2.0,2.238,0.016,-9.746
5.13,2.0,9.986,0.079,-0.528
4.37,2.0,9.945,0.085,-1.048
5.8,2.0,8.903,0.084,4.553
5.04,2.0,8.684,0.088,-4.958
6.33,2.0,9.878,0.104,1.557
5.7,2.0,9.89,0.108,1.476
4.55,2.0,9.544,0.156,-2.98
5.9,2.0,4.65,0.086,8.853
4.99,2.0,9.83,0.193,-1.824
4.61,2.0,9.948,0.232,-0.995
5.55,2.0,9.725,0.242,2.317
5.57,2.0,4.351,0.122,9.003
6.18,2.0,9.984,0.357,-0.427
2.07,-0.038,8.733,0.32,4.862
5.57,2.0,9.492,0.375,3.125
2.28,0.38,5.124,0.205,8.585
5.42,2.0,8.823,0.36,-4.693
3.88,2.0,6.972,0.286,-7.163
5.54,2.0,9.802,0.43,1.933
5.29,2.0,1.352,0.059,-9.908
5.01,2.0,6.93,0.312,7.202
4.89,2.0,9.626,0.474,-2.667
5.41,2.0,8.835,0.447,-4.664
5.24,2.0,8.167,0.418,-5.755
2.83,2.0,9.635,0.557,2.619
5.71,2.0,7.53,0.445,6.565
4.79,2.0,9.365,0.598,3.454
4.44,2.0,9.44,0.604,-3.245
6.02,2.0,9.861,0.645,1.533
6.12,2.0,9.871,0.715,1.433
4.61,2.0,7.785,0.582,6.25
4.51,2.0,7.983,0.64,5.988
6.1,2.0,7.195,0.588,6.92
3.56,2.0,9.846,0.837,-1.534
4.23,2.0,4.229,0.371,-9.054
5.38,2.0,9.858,0.888,1.425
5.5,2.0,3.468,0.313,-9.374
5.16,2.0,7.85,0.725,6.152
5.18,2.0,8.709,0.82,-4.845
6.25,2.0,9.678,0.95,2.331
6.39,2.0,9.725,0.973,-2.115
5.38,2.0,4.693,0.51,8.816
5.77,2.0,9.933,1.106,0.338
2.82,2.0,2.192,0.246,-9.754
3.93,2.0,7.185,0.825,-6.906
2.4,1.083,7.347,0.846,-6.731
6.16,2.0,9.933,1.159,-0.009
4.86,2.0,8.324,1.019,-5.447
5.01,2.0,9.445,1.162,3.072
6.05,2.0,9.518,1.178,2.831
5.72,2.0,9.891,1.304,-0.69
5.2,2.0,8.607,1.138,4.962
4.76,2.0,6.525,0.9,-7.525
6.18,2.0,3.947,0.545,9.172
4.36,2.0,4.503,0.624,-8.907
4.53,2.0,4.502,0.624,-8.907
4.74,2.0,5.748,0.802,8.143
5.69,2.0,9.827,1.398,1.211
5.38,2.0,9.285,1.329,3.468
4.17,2.0,4.503,0.653,8.905
6.11,2.0,3.178,0.466,-9.47
6.47,2.0,3.903,0.591,9.188
5.2,2.0,9.863,1.529,-0.627
5.94,2.0,9.88,1.545,-0.088
5.51,2.0,6.61,1.038,-7.432
5.89,2.0,9.525,1.542,2.627
4.34,2.0,8.21,1.333,5.551
3.69,2.0,5.816,0.946,8.08
4.34,2.0,8.597,1.46,4.896
3.27,2.0,8.458,1.466,5.13
5.88,2.0,9.183,1.593,3.625
5.36,2.0,9.167,1.613,3.655
2.24,1.17,5.428,0.969,8.342
5.3,2.0,7.597,1.378,6.355
4.59,2.0,6.823,1.244,-7.204
5.72,2.0,5.428,1.0,-8.339
4.8,2.0,6.252,1.161,7.718
5.38,2.0,4.081,0.765,-9.097
6.05,2.0,7.694,1.451,-6.22
4.36,2.0,5.282,1.011,-8.43
4.95,2.0,6.695,1.285,7.316
2.04,1.019,9.34,1.798,-3.088
4.77,2.0,9.647,1.883,-1.841
5.9,2.0,7.689,1.502,-6.214
4.48,2.0,6.528,1.29,7.465
6.15,2.0,9.557,1.922,-2.229
5.64,2.0,2.539,0.513,9.659
5.36,2.0,9.439,1.944,2.668
5.51,2.0,9.577,1.993,2.075
6.11,2.0,9.222,1.928,3.351
4.08,2.0,8.923,1.87,4.11
5.98,2.0,9.719,2.039,1.174
5.42,2.0,2.557,0.541,9.652
6.51,2.0,9.137,1.938,3.572
5.92,2.0,9.7,2.074,1.271
5.09,2.0,2.543,0.547,-9.656
4.44,2.0,9.69,2.09,1.32
4.9,2.0,6.155,1.332,7.768
5.07,2.0,9.348,2.029,2.914
3.46,2.0,5.205,1.132,8.463
4.53,2.0,7.361,1.626,6.571
5.55,2.0,8.644,1.912,4.65
5.17,2.0,9.594,2.133,-1.847
5.24,2.0,6.142,1.381,-7.77
6.67,2.0,3.41,0.794,-9.367
4.78,2.0,9.732,2.292,-0.2
6.15,2.0,9.608,2.32,-1.52
5.8,2.0,9.178,2.228,3.287
5.46,2.0,8.899,2.176,4.008
4.83,2.0,5.007,1.225,8.569
5.45,2.0,3.397,0.831,-9.368
6.08,2.0,8.629,2.151,4.572
5.35,2.0,9.516,2.374,-1.954
4.62,2.0,4.967,1.254,8.588
2.15,-0.046,4.742,1.198,8.722
3.86,2.0,7.587,1.918,6.225
4.4,2.0,8.892,2.267,3.974
5.44,2.0,8.47,2.184,4.847
4.3,2.0,8.432,2.205,-4.903
5.62,2.0,9.483,2.485,-1.973
5.59,2.0,7.504,2.056,-6.282
5.5,2.0,8.207,2.293,-5.233
5.5,2.0,8.181,2.3,5.27
5.95,2.0,7.226,2.035,6.606
4.27,2.0,9.534,2.686,1.373
5.4,2.0,9.59,2.707,-0.843
6.06,2.0,9.612,2.748,0.239
6.03,2.0,9.555,2.78,0.986
5.64,2.0,9.275,2.707,2.579
6.09,2.0,9.448,2.781,-1.733
5.33,2.0,8.926,2.631,3.661
5.55,2.0,8.927,2.631,3.659
6.35,2.0,9.555,2.822,0.856
7.26,2.0,9.555,2.824,0.856
5.58,2.0,9.446,2.802,-1.709
3.32,2.0,6.573,1.95,-7.28
6.14,2.0,9.337,2.791,2.242
5.36,2.0,4.527,1.369,-8.811
5.71,2.0,9.427,2.872,-1.7
5.21,2.0,7.166,2.184,-6.624
5.56,2.0,8.944,2.732,3.541
5.04,2.0,6.886,2.106,6.939
6.23,2.0,8.109,2.48,5.301
5.17,2.0,5.494,1.685,8.184
5.51,2.0,9.512,2.925,0.985
3.94,2.0,5.449,1.676,-8.216
3.46,2.0,9.405,2.902,-1.768
4.26,2.0,6.479,2.027,7.342
2.07,1.576,7.756,2.435,5.824
5.57,2.0,8.984,2.825,3.364
6.4,2.0,9.42,2.979,-1.548
5.67,2.0,7.075,2.242,6.702
5.97,2.0,9.521,3.028,0.427
5.32,2.0,3.449,1.098,9.322
4.34,2.0,5.442,1.744,8.207
5.15,2.0,8.126,2.605,5.214
5.8,2.0,7.531,2.417,6.119
4.66,2.0,8.884,2.863,3.589
4.51,2.0,8.233,2.662,5.014
5.57,2.0,4.018,1.3,9.064
5.93,2.0,9.507,3.076,-0.393
5.21,2.0,9.404,3.134,1.318
4.67,2.0,8.627,2.876,4.16
6.44,2.0,9.404,3.135,1.319
5.97,2.0,9.108,3.053,2.779
5.14,2.0,9.387,3.159,-1.379
6.04,2.0,9.403,3.178,1.218
5.7,2.0,9.47,3.206,-0.17
4.97,2.0,6.632,2.257,-7.136
4.25,2.0,3.409,1.17,-9.328
5.42,2.0,9.438,3.278,-0.436
5.13,2.0,9.411,3.323,0.63
4.74,2.0,8.36,3.021,4.581
5.87,2.0,9.399,3.412,-0.089
4.95,2.0,4.947,1.802,8.502
6.33,2.0,4.015,1.483,9.038
5.23,2.0,8.225,3.039,4.808
4.87,2.0,6.558,2.463,7.136
6.5,2.0,9.358,3.526,-0.078
5.6,2.0,7.389,2.824,6.117
3.6,2.0,9.24,3.548,-1.423
6.21,2.0,9.244,3.55,-1.393
4.9,2.0,9.01,3.532,-2.52
2.66,2.0,4.62,1.816,8.681
4.72,2.0,3.466,1.364,9.28
5.35,2.0,8.784,3.471,3.284
5.5,2.0,8.774,3.487,3.295
5.51,2.0,9.05,3.604,-2.259
4.83,2.0,6.513,2.62,7.121
3.41,2.0,6.741,2.736,-6.86
5.11,2.0,8.594,3.542,-3.686
6.01,2.0,8.771,3.628,3.149
5.27,2.0,6.299,2.612,7.314
4.84,2.0,9.183,3.812,1.07
5.82,2.0,3.113,1.309,9.413
3.93,2.0,6.039,2.54,-7.555
3.62,2.0,8.885,3.748,2.646
4.68,2.0,4.692,2.038,8.592
5.62,2.0,8.819,3.863,-2.702
6.23,2.0,8.842,3.926,2.531
5.41,2.0,8.808,3.921,-2.656
5.69,2.0,7.917,3.531,-4.986
4.1,2.0,6.841,3.075,6.614
5.54,2.0,8.912,4.019,2.103
0.45,-0.158,4.927,2.238,-8.409
3.59,2.0,6.014,2.741,7.504
5.28,2.0,2.652,1.216,9.565
5.01,2.0,6.485,3.002,6.995
5.98,2.0,8.7,4.042,2.824
5.76,2.0,5.044,2.347,-8.309
4.96,2.0,6.876,3.227,6.505
6.33,2.0,1.702,0.806,-9.821
4.45,2.0,8.995,4.263,0.956
5.25,2.0,7.625,3.643,-5.347
5.57,2.0,3.372,1.615,9.275
5.52,2.0,5.333,2.558,-8.063
5.24,2.0,8.458,4.057,3.464
5.18,2.0,2.989,1.441,9.434
5.78,2.0,4.425,2.142,8.708
4.01,2.0,5.698,2.769,7.737
3.49,2.0,8.641,4.218,-2.746
6.27,2.0,8.425,4.151,3.434
4.26,2.0,8.847,4.382,1.592
5.29,2.0,8.114,4.03,-4.234
5.04,2.0,5.319,2.655,-8.041
6.05,2.0,1.521,0.773,-9.853
5.86,2.0,8.519,4.349,2.916
4.66,2.0,8.724,4.522,-1.854
5.83,2.0,8.205,4.278,3.791
3.74,2.0,8.697,4.598,-1.794
5.53,2.0,5.046,2.683,8.206
5.7,2.0,5.579,2.971,7.749
3.42,2.0,7.659,4.119,4.936
5.42,2.0,6.671,3.595,6.525
3.88,2.0,8.304,4.487,3.304
4.61,2.0,8.784,4.747,0.556
4.39,2.0,6.076,3.287,-7.23
5.12,2.0,6.474,3.529,-6.756
3.35,2.0,3.894,2.123,8.963
2.64,2.0,8.202,4.483,3.552
4.68,2.0,3.335,1.828,-9.249
6.15,2.0,1.767,0.978,-9.794
5.76,2.0,8.019,4.438,4.0
3.69,2.0,5.432,3.009,-7.838
4.97,2.0,3.179,1.762,9.316
5.69,2.0,6.959,3.864,6.053
4.92,2.0,8.066,4.502,-3.831
5.09,2.0,8.299,4.665,3.06
4.79,2.0,7.977,4.51,4.003
5.7,2.0,5.679,3.231,7.57
2.86,2.0,4.136,2.358,-8.794
5.43,2.0,8.099,4.665,-3.555
3.99,2.0,8.081,4.666,-3.596
5.89,2.0,8.644,4.998,0.54
5.34,2.0,7.476,4.371,-5.0
5.15,2.0,6.128,3.599,-7.036
4.49,2.0,2.819,1.66,9.45
3.82,2.0,8.605,5.071,0.482
4.99,2.0,5.001,2.955,8.14
6.0,2.0,3.647,2.167,9.056
5.5,2.0,7.185,4.273,5.488
5.59,2.0,3.715,2.21,9.018
5.42,2.0,8.59,5.12,0.022
3.95,2.0,2.593,1.549,9.533
5.64,2.0,7.715,4.62,4.374
5.96,2.0,8.576,5.143,-0.059
2.1,1.37,6.339,3.805,6.734
4.68,2.0,7.466,4.508,-4.893
5.27,2.0,1.882,1.143,9.755
5.22,2.0,2.049,1.25,9.708
5.03,2.0,7.857,4.841,3.851
6.01,2.0,7.666,4.735,4.337
2.01,1.151,7.797,4.833,3.982
4.78,2.0,6.687,4.198,6.137
4.98,2.0,7.596,4.813,4.374
3.0,2.0,6.919,4.388,5.734
5.68,2.0,7.936,5.086,3.338
6.09,2.0,6.535,4.198,6.299
6.82,2.0,6.534,4.199,6.299
6.01,2.0,7.559,4.871,4.374
5.64,2.0,8.308,5.362,1.49
6.24,2.0,7.161,4.624,5.229
6.38,2.0,4.495,2.905,8.448
5.94,2.0,8.392,5.429,-0.319
4.94,2.0,7.233,4.714,5.046
5.65,2.0,8.361,5.47,-0.418
5.23,2.0,7.801,5.105,3.618
5.27,2.0,7.191,4.71,-5.109
4.36,2.0,8.263,5.418,1.538
5.72,2.0,8.066,5.291,2.635
4.84,2.0,5.988,3.934,6.976
5.31,2.0,5.246,3.459,7.779
5.57,2.0,3.143,2.085,-9.261
6.05,2.0,3.317,2.206,9.172
5.67,2.0,3.144,2.111,-9.255
5.57,2.0,7.517,5.057,4.233
5.79,2.0,7.47,5.028,4.35
5.25,2.0,6.926,4.669,5.499
3.56,2.0,5.152,3.492,-7.827
5.51,2.0,8.214,5.592,-1.118
4.84,2.0,6.834,4.655,5.624
4.03,2.0,6.859,4.684,5.57
5.75,2.0,4.379,3.01,8.471
5.99,2.0,4.425,3.043,8.435
5.58,2.0,7.746,5.33,3.404
5.29,2.0,7.212,5.001,4.793
5.31,2.0,5.559,3.866,7.359
6.47,2.0,8.197,5.705,-0.519
5.57,2.0,5.233,3.697,7.677
4.08,2.0,2.965,2.11,-9.314
5.29,2.0,8.142,5.805,0.069
5.42,2.0,8.135,5.814,-0.154
5.16,2.0,4.566,3.267,8.275
5.19,2.0,7.435,5.33,-4.038
5.99,2.0,2.286,1.644,-9.595
5.19,2.0,5.193,3.787,7.661
5.48,2.0,7.931,5.806,1.841
5.36,2.0,3.995,2.927,-8.687
6.34,2.0,8.056,5.905,-0.485
6.25,2.0,4.434,3.259,8.349
4.73,2.0,5.143,3.793,7.692
4.88,2.0,7.856,5.81,-2.129
4.24,2.0,5.392,4.026,-7.397
5.55,2.0,6.799,5.099,5.27
6.16,2.0,5.075,3.819,7.724
5.13,2.0,6.635,5.001,-5.565
4.3,2.0,7.895,5.958,1.471
5.29,2.0,6.936,5.234,4.95
5.89,2.0,6.903,5.24,4.99
4.46,2.0,3.058,2.327,9.232
6.14,2.0,7.446,5.747,3.396
6.21,2.0,7.535,5.829,3.041
5.27,2.0,1.491,1.161,-9.82
1.97,0.636,0.101,0.079,9.999
4.74,2.0,7.601,5.943,-2.629
5.15,2.0,6.361,4.974,5.899
5.36,2.0,7.875,6.161,-0.181
6.0,2.0,7.587,5.976,2.594
5.91,2.0,6.458,5.096,-5.686
4.96,2.0,6.899,5.48,-4.73
5.74,2.0,7.733,6.191,-1.367
5.38,2.0,6.395,5.169,5.691
4.87,2.0,7.738,6.259,0.975
5.53,2.0,7.699,6.235,-1.363
5.74,2.0,6.724,5.452,-5.007
5.64,2.0,7.572,6.166,2.156
7.1,2.0,7.039,5.749,4.17
5.78,2.0,6.377,5.209,-5.675
6.48,2.0,7.038,5.751,4.17
5.3,2.0,4.703,3.856,-7.938
5.65,2.0,7.711,6.339,-0.592
5.84,2.0,6.662,5.507,-5.029
5.45,2.0,7.135,5.925,3.74
4.08,2.0,7.675,6.41,0.057
4.83,2.0,7.509,6.276,-2.057
4.12,2.0,2.841,2.375,-9.289
4.74,2.0,5.616,4.704,-6.806
5.21,2.0,4.432,3.741,-8.146
4.11,2.0,5.866,4.952,-6.409
5.3,2.0,6.805,5.744,4.549
5.72,2.0,7.625,6.468,-0.121
4.91,2.0,5.803,4.967,6.454
5.74,2.0,7.135,6.114,3.422
5.4,2.0,4.796,4.116,-7.75
5.76,2.0,4.333,3.735,8.202
3.47,2.0,7.555,6.527,0.565
4.65,2.0,6.696,5.792,4.65
5.43,2.0,5.4,4.698,6.984
4.24,2.0,7.324,6.374,-2.395
4.1,2.0,4.925,4.289,7.573
6.4,2.0,7.179,6.258,3.051
5.78,2.0,7.264,6.345,2.641
4.27,2.0,7.403,6.489,1.756
5.17,2.0,7.343,6.437,2.155
4.47,2.0,7.124,6.253,-3.185
5.73,2.0,3.325,2.928,-8.965
4.83,2.0,2.857,2.518,-9.247
6.51,2.0,6.057,5.433,-5.814
4.52,2.0,6.486,5.836,4.886
5.83,2.0,7.041,6.37,3.137
5.25,2.0,3.382,3.073,-8.895
4.45,2.0,6.247,5.679,-5.359
5.26,2.0,7.053,6.423,3.001
6.14,2.0,6.71,6.14,-4.156
5.39,2.0,6.516,5.966,-4.686
3.61,2.0,6.554,6.005,4.58
5.92,2.0,5.971,5.483,-5.856
4.76,2.0,1.896,1.745,-9.662
4.22,2.0,5.771,5.315,6.2
5.48,2.0,5.973,5.506,-5.832
3.77,2.0,4.122,3.801,8.28
4.76,2.0,6.854,6.338,-3.584
5.52,2.0,7.076,6.569,2.602
4.56,2.0,5.998,5.57,5.744
5.93,2.0,5.692,5.381,-6.217
5.34,2.0,5.696,5.392,6.203
3.93,2.0,4.385,4.17,7.961
5.76,2.0,6.834,6.588,3.145
3.89,2.0,7.094,6.876,-1.547
5.58,2.0,6.828,6.619,3.094
5.1,2.0,6.071,5.929,5.289
5.44,2.0,6.54,6.393,-4.045
5.8,2.0,6.671,6.56,3.53
5.82,2.0,6.533,6.425,-4.004
2.88,2.0,5.433,5.351,-6.469
4.68,2.0,5.473,5.414,6.383
4.98,2.0,3.108,3.076,-8.993
4.94,2.0,5.803,5.756,5.762
4.63,2.0,6.609,6.564,3.639
5.69,2.0,6.405,6.383,-4.27
5.56,2.0,7.074,7.055,-0.43
4.71,2.0,6.994,6.977,1.548
6.11,2.0,7.036,7.088,-0.502
5.75,2.0,6.972,7.043,-1.333
5.88,2.0,6.194,6.282,-4.709
5.91,2.0,6.277,6.383,4.456
5.51,2.0,2.175,2.218,-9.505
2.54,1.63,6.983,7.123,0.713
5.62,2.0,6.977,7.123,0.759
4.08,2.0,6.41,6.546,-4.007
5.32,2.0,6.924,7.09,-1.337
5.12,2.0,3.507,3.619,-8.637
5.26,2.0,6.877,7.138,-1.323
2.91,2.0,4.116,4.292,8.039
3.32,2.0,5.382,5.631,6.272
5.45,2.0,6.241,6.545,4.267
4.77,2.0,3.787,3.974,8.359
6.13,2.0,6.508,6.944,3.07
2.09,-0.003,5.146,5.527,6.555
6.24,2.0,6.446,6.933,3.222
4.05,2.0,4.397,4.759,7.617
3.79,2.0,4.801,5.216,7.054
5.74,2.0,5.915,6.434,4.86
4.61,2.0,5.173,5.709,6.376
4.35,2.0,6.31,6.985,3.375
3.8,2.0,5.851,6.502,-4.846
5.78,2.0,5.941,6.612,4.58
5.07,2.0,6.665,7.452,-0.209
4.87,2.0,6.157,7.014,3.591
4.8,2.0,6.488,7.453,-1.533
5.51,2.0,1.433,1.648,-9.759
6.14,2.0,6.458,7.468,-1.591
5.53,2.0,2.995,3.499,-8.876
5.49,2.0,4.675,5.463,6.95
5.24,2.0,2.995,3.513,-8.871
4.86,2.0,5.988,7.034,-3.829
5.62,2.0,6.481,7.614,-0.162
5.16,2.0,4.142,4.877,7.685
5.05,2.0,4.142,4.899,7.671
4.84,2.0,6.438,7.629,0.588
3.7,2.0,5.985,7.102,-3.707
4.26,2.0,4.698,5.594,-6.829
5.91,2.0,5.726,6.819,4.551
6.14,2.0,5.781,6.916,4.331
5.7,2.0,6.377,7.676,0.641
5.27,2.0,5.957,7.177,3.608
4.96,2.0,4.641,5.602,6.862
5.55,2.0,5.631,6.843,4.634
5.1,2.0,5.925,7.236,3.542
5.5,2.0,5.707,7.067,4.183
1.79,0.481,4.05,5.016,7.645
6.07,2.0,5.869,7.276,3.552
3.61,2.0,6.188,7.697,1.569
6.39,2.0,5.043,6.336,-5.867
3.73,2.0,6.096,7.745,1.691
5.71,2.0,5.013,6.391,-5.833
6.49,2.0,4.985,6.391,-5.857
6.03,2.0,5.661,7.275,3.876
4.67,2.0,3.967,5.141,7.605
4.71,2.0,2.779,3.602,-8.905
5.14,2.0,5.955,7.789,1.966
4.36,2.0,4.06,5.319,7.431
4.74,2.0,6.042,7.919,-0.885
4.14,2.0,5.904,7.755,2.239
5.3,2.0,4.166,5.55,7.2
5.76,2.0,5.917,7.895,1.629
3.72,2.0,5.904,7.902,-1.643
4.26,2.0,5.536,7.467,-3.687
5.95,2.0,5.4,7.327,4.141
5.24,2.0,5.597,7.724,-3.002
4.32,2.0,3.907,5.401,7.454
4.29,2.0,5.847,8.112,0.07
6.72,2.0,5.371,7.458,3.941
4.57,2.0,4.455,6.194,-6.465
6.01,2.0,5.105,7.21,-4.686
5.97,2.0,5.743,8.128,-0.98
5.55,2.0,5.733,8.176,0.533
5.53,2.0,5.689,8.174,-0.908
6.11,2.0,5.159,7.421,4.278
4.99,2.0,4.799,6.999,-5.29
5.68,2.0,5.322,7.766,3.371
4.97,2.0,4.686,6.843,5.587
4.59,2.0,4.481,6.571,-6.062
3.01,2.0,3.783,5.552,7.407
3.52,2.0,5.538,8.152,-1.696
6.13,2.0,5.273,7.802,3.365
3.84,2.0,2.377,3.531,-9.049
3.84,2.0,4.718,7.015,5.342
5.24,2.0,5.572,8.301,-0.203
5.45,2.0,5.07,7.574,4.113
3.72,2.0,5.075,7.586,4.085
5.56,2.0,5.558,8.313,-0.052
5.66,2.0,5.036,7.549,4.201
3.77,2.0,4.086,6.126,6.766
4.3,2.0,5.05,7.573,4.142
5.34,2.0,5.5,8.285,1.054
3.87,2.0,5.033,7.592,4.126
5.76,2.0,5.023,7.583,4.156
6.43,2.0,5.02,7.587,4.151
4.43,2.0,5.392,8.157,-2.096
4.14,2.0,5.033,7.628,4.059
5.91,2.0,4.797,7.279,-4.9
4.22,2.0,5.043,7.681,-3.947
3.26,2.0,1.487,2.273,-9.624
2.85,2.0,4.989,7.644,4.084
5.24,2.0,4.992,7.662,-4.047
5.52,2.0,4.711,7.249,-5.025
5.08,2.0,5.334,8.235,1.933
3.62,2.0,4.935,7.684,4.076
5.05,2.0,4.931,7.679,4.089
4.17,2.0,4.352,6.795,-5.906
5.14,2.0,4.516,7.057,5.46
4.59,2.0,1.716,2.702,9.474
5.66,2.0,5.265,8.425,1.138
5.48,2.0,5.25,8.459,-0.934
5.11,2.0,4.305,7.001,-5.697
4.64,2.0,4.76,7.746,-4.165
2.84,2.0,4.432,7.243,5.282
4.46,2.0,5.207,8.522,-0.515
5.28,2.0,3.248,5.438,7.738
5.62,2.0,4.729,7.938,3.823
6.05,2.0,4.698,7.902,3.936
2.9,2.0,3.892,6.597,6.429
2.97,2.0,4.934,8.379,-2.336
4.56,2.0,2.416,4.132,-8.78
3.98,2.0,4.087,7.006,5.848
4.62,2.0,4.57,7.909,-4.07
3.41,2.0,4.857,8.47,2.163
4.48,2.0,2.319,4.054,-8.842
4.97,2.0,2.394,4.202,-8.753
5.28,2.0,4.94,8.69,-0.27
3.91,2.0,4.854,8.681,1.043
5.32,2.0,4.836,8.701,0.947
5.46,2.0,4.413,7.99,4.084
4.36,2.0,4.468,8.118,3.759
5.9,2.0,4.447,8.135,3.748
4.25,2.0,3.03,5.615,7.7
5.18,2.0,4.208,7.799,4.633
5.21,2.0,4.139,7.705,4.848
6.07,2.0,3.705,6.988,6.119
5.52,2.0,3.679,6.964,6.162
3.96,2.0,3.142,5.95,7.398
5.51,2.0,4.38,8.34,3.356
5.44,2.0,4.569,8.813,-1.205
5.39,2.0,4.104,7.954,4.459
4.93,2.0,3.408,6.605,-6.69
5.71,2.0,4.544,8.856,0.962
4.04,2.0,4.513,8.844,-1.191
5.29,2.0,4.439,8.86,1.343
4.84,2.0,4.406,8.831,1.61
3.85,2.0,3.301,6.62,-6.729
4.87,2.0,4.376,8.814,-1.78
3.33,2.0,2.054,4.14,-8.868
4.67,2.0,3.367,6.82,6.492
4.12,2.0,2.939,5.952,7.479
4.43,2.0,4.372,8.895,-1.33
4.27,2.0,4.349,8.871,1.546
6.31,2.0,4.235,8.661,2.656
4.26,2.0,2.729,5.597,-7.825
4.44,2.0,2.228,4.593,-8.599
4.93,2.0,4.058,8.437,3.515
5.88,2.0,1.943,4.059,-8.93
3.55,2.0,3.581,7.499,-5.563
5.67,2.0,0.732,1.534,-9.855
4.6,2.0,2.744,5.769,7.694
5.64,2.0,3.989,8.4,3.678
5.5,2.0,3.963,8.443,3.607
5.34,2.0,3.939,8.41,3.709
3.65,2.0,4.078,8.724,2.694
5.58,2.0,4.102,8.792,2.425
4.97,2.0,3.741,8.056,4.594
4.93,2.0,3.467,7.469,5.674
5.26,2.0,4.057,8.761,2.604
5.78,2.0,0.613,1.33,-9.892
4.8,2.0,2.867,6.258,7.254
5.24,2.0,1.86,4.075,-8.941
5.72,2.0,4.02,8.827,2.432
5.38,2.0,3.718,8.214,4.325
3.77,2.0,3.919,8.692,3.014
5.64,2.0,3.916,8.737,2.887
5.17,2.0,4.071,9.11,-0.653
5.1,2.0,4.017,9.009,1.644
6.34,2.0,3.707,8.326,4.115
3.97,2.0,3.37,7.573,-5.594
4.8,2.0,3.877,8.717,2.998
5.73,2.0,3.351,7.569,5.611
5.77,2.0,3.353,7.586,5.586
4.21,2.0,3.713,8.475,3.794
5.27,2.0,3.714,8.481,3.778
4.3,2.0,3.813,8.717,3.078
6.44,2.0,3.849,8.812,2.746
4.28,2.0,3.664,8.458,3.877
4.48,2.0,3.827,8.838,2.692
4.69,2.0,3.833,8.88,2.54
5.53,2.0,3.623,8.462,3.907
5.9,2.0,3.764,8.908,2.545
4.96,2.0,3.732,8.84,2.817
3.84,2.0,3.734,8.86,2.75
3.53,2.0,3.667,8.704,3.285
3.4,2.0,3.733,8.865,2.735
5.02,2.0,3.774,8.981,2.258
5.58,2.0,3.68,8.899,2.696
5.4,2.0,3.693,8.985,2.373
5.47,2.0,3.659,8.905,2.705
5.07,2.0,2.684,6.548,-7.065
6.01,2.0,3.609,8.917,2.731
4.91,2.0,3.751,9.27,-0.008
5.78,2.0,2.206,5.462,8.081
6.09,2.0,2.695,6.791,6.828
4.49,2.0,3.199,8.07,-4.965
4.65,2.0,3.549,8.991,2.562
5.71,2.0,3.643,9.238,-1.173
3.3,2.0,2.1,5.331,-8.196
5.2,2.0,3.619,9.211,-1.432
3.81,2.0,3.102,8.033,-5.085
4.25,2.0,3.541,9.184,1.764
0.87,1.538,3.439,8.95,2.842
3.93,2.0,3.565,9.325,-0.585
4.25,2.0,2.673,7.026,6.595
5.22,2.0,3.515,9.352,-0.432
5.78,2.0,3.36,9.005,2.762
4.27,2.0,3.413,9.147,2.166
3.86,2.0,3.386,9.079,-2.471
5.08,2.0,3.324,9.03,2.723
4.67,2.0,3.318,9.026,2.743
5.07,2.0,2.057,5.644,7.995
5.36,2.0,2.036,5.593,8.036
5.45,2.0,3.341,9.186,2.113
4.32,2.0,3.204,8.855,-3.366
4.44,2.0,2.53,7.004,-6.674
5.04,2.0,2.659,7.515,-6.038
4.27,2.0,3.064,8.683,3.9
5.3,2.0,2.104,6.005,-7.714
5.3,2.0,2.4,6.862,6.866
5.53,2.0,1.076,3.085,-9.451
6.18,2.0,3.001,8.622,4.082
5.98,2.0,3.235,9.338,-1.529
5.78,2.0,3.216,9.352,-1.479
5.28,2.0,1.634,4.768,-8.637
4.01,2.0,3.188,9.461,-0.568
5.49,2.0,2.972,9.093,-2.913
6.35,2.0,2.682,8.233,-5.003
5.29,2.0,1.694,5.214,8.364
5.76,2.0,2.944,9.134,-2.812
6.09,2.0,2.903,9.169,2.74
3.19,2.0,2.992,9.465,1.212
4.89,2.0,2.389,7.567,6.086
5.03,2.0,2.88,9.16,-2.793
4.35,2.0,2.946,9.43,1.547
5.58,2.0,1.767,5.685,-8.035
6.42,2.0,1.767,5.686,-8.034
3.68,2.0,2.943,9.507,0.977
5.08,2.0,2.792,9.043,3.229
4.71,2.0,2.812,9.275,2.462
4.79,2.0,2.323,7.674,5.977
4.36,2.0,2.873,9.531,-0.95
5.33,2.0,2.863,9.571,0.438
4.26,2.0,1.139,3.848,9.16
3.71,2.0,2.827,9.583,0.426
5.18,2.0,2.752,9.408,1.981
4.64,2.0,2.759,9.449,1.762
5.52,2.0,1.593,5.471,8.217
5.47,2.0,0.725,2.496,-9.656
4.06,2.0,2.665,9.351,2.337
5.5,2.0,2.728,9.578,-0.901
2.69,2.0,2.272,8.057,5.471
4.43,2.0,1.598,5.693,8.065
5.79,2.0,2.45,8.805,4.059
5.79,2.0,2.415,8.732,4.234
4.47,2.0,2.648,9.638,0.299
5.84,2.0,0.345,1.264,-9.914
4.93,2.0,2.067,7.616,6.142
6.09,2.0,1.558,5.791,8.003
5.39,2.0,2.553,9.503,-1.782
4.78,2.0,2.529,9.428,-2.171
5.95,2.0,1.99,7.467,6.347
6.46,2.0,1.98,7.44,6.382
4.8,2.0,2.508,9.6,-1.249
3.03,2.0,1.807,6.985,6.924
3.69,2.0,1.872,7.302,6.571
5.3,2.0,0.792,3.104,-9.473
5.74,2.0,2.279,8.933,-3.874
5.37,2.0,1.615,6.338,-7.564
4.62,2.0,2.285,9.013,3.68
4.03,2.0,1.206,4.783,8.699
4.55,2.0,1.956,7.904,-5.805
6.32,2.0,1.949,7.883,-5.836
4.65,2.0,2.309,9.36,2.656
5.05,2.0,1.542,6.298,-7.613
3.19,2.0,2.18,8.987,-3.806
4.71,2.0,1.266,5.226,-8.431
5.22,2.0,1.2,5.013,8.569
6.08,2.0,1.197,5.006,8.574
3.18,2.0,1.739,7.316,6.591
4.98,2.0,1.432,6.044,7.837
5.12,2.0,2.295,9.699,-0.812
4.91,2.0,2.154,9.227,3.197
6.15,2.0,2.234,9.607,1.646
5.28,2.0,2.116,9.13,3.489
2.78,2.0,2.247,9.704,-0.887
5.33,2.0,2.23,9.636,1.478
5.84,2.0,2.093,9.052,3.698
5.5,2.0,2.046,8.884,4.11
5.11,2.0,2.212,9.721,-0.777
4.25,2.0,2.175,9.641,-1.522
5.43,2.0,2.161,9.613,1.707
4.81,2.0,2.097,9.401,2.689
4.45,2.0,2.022,9.575,-2.057
3.29,2.0,1.958,9.401,-2.791
4.36,2.0,1.975,9.544,-2.24
4.46,2.0,2.021,9.781,0.499
4.82,2.0,1.58,7.667,6.223
6.49,2.0,0.924,4.494,8.885
4.81,2.0,0.777,3.799,-9.218
0.18,-0.03,1.951,9.704,-1.427
5.01,2.0,1.627,8.257,5.401
6.26,2.0,1.787,9.079,3.792
5.52,2.0,1.868,9.625,1.967
0.08,0.795,1.305,6.823,7.193
4.81,2.0,1.513,8.061,-5.721
3.59,2.0,1.826,9.759,-1.192
4.54,2.0,1.516,8.212,5.501
6.15,2.0,1.504,8.176,5.558
4.69,2.0,1.357,7.528,6.441
5.34,2.0,1.77,9.832,0.453
4.96,2.0,1.638,9.12,3.762
5.44,2.0,1.119,6.247,-7.728
6.49,2.0,1.461,8.162,5.59
6.13,2.0,0.929,5.199,8.491
4.29,2.0,1.709,9.586,-2.28
5.29,2.0,1.697,9.621,-2.133
5.05,2.0,1.44,8.169,5.586
4.72,2.0,1.661,9.861,-0.067
5.22,2.0,1.236,7.351,6.666
6.1,2.0,0.911,5.498,-8.303
4.99,2.0,1.612,9.85,0.618
6.45,2.0,1.392,8.64,4.838
5.24,2.0,0.852,5.298,8.438
5.25,2.0,1.539,9.583,-2.407
6.09,2.0,1.514,9.458,2.873
4.13,2.0,1.552,9.785,-1.359
5.0,2.0,1.476,9.428,2.988
3.35,2.0,1.542,9.871,-0.418
5.07,2.0,1.543,9.879,-0.156
5.02,2.0,1.221,7.851,6.072
4.89,2.0,1.531,9.877,0.322
6.26,2.0,0.936,6.041,-7.914
1.64,-0.224,1.506,9.824,1.106
6.23,2.0,1.412,9.474,2.874
1.65,-0.13,1.287,8.684,4.788
5.14,2.0,0.756,5.108,-8.564
4.59,2.0,1.44,9.881,0.54
5.4,2.0,1.358,9.415,3.084
4.88,2.0,1.306,9.184,3.736
5.08,2.0,1.16,8.162,5.661
5.52,2.0,1.349,9.524,2.735
5.77,2.0,1.328,9.458,2.964
2.81,2.0,1.291,9.261,-3.544
5.47,2.0,1.21,8.971,4.25
4.71,2.0,1.317,9.911,-0.191
5.43,2.0,0.588,4.491,8.915
6.18,2.0,0.118,0.901,-9.959
4.2,2.0,1.264,9.865,1.036
5.53,2.0,1.174,9.27,-3.561
3.86,2.0,1.02,8.08,-5.803
5.46,2.0,1.249,9.905,0.574
5.18,2.0,0.289,2.344,-9.717
4.62,2.0,1.212,9.845,-1.271
2.25,2.0,1.218,9.925,-0.052
4.32,2.0,1.146,9.408,3.189
6.44,2.0,0.647,5.375,8.408
4.71,2.0,1.005,8.403,5.328
2.58,0.211,1.13,9.453,-3.061
5.67,2.0,1.093,9.418,3.18
3.76,2.0,0.53,4.589,-8.869
5.6,2.0,1.101,9.627,2.471
5.32,2.0,1.117,9.916,0.657
4.39,2.0,1.081,9.804,1.649
3.39,2.0,1.066,9.792,1.725
4.98,2.0,1.072,9.898,-0.939
5.13,2.0,1.072,9.898,-0.939
6.71,2.0,1.071,9.898,-0.939
4.98,2.0,1.067,9.898,-0.944
4.58,2.0,1.068,9.907,-0.843
2.75,2.0,1.064,9.89,-1.03
5.37,2.0,0.976,9.08,4.074
6.04,2.0,0.206,1.928,-9.81
5.24,2.0,1.056,9.908,-0.847
1.69,-0.184,1.036,9.944,-0.21
4.09,2.0,0.993,9.819,1.615
5.53,2.0,0.955,9.513,2.93
5.65,2.0,0.165,1.647,-9.862
6.15,2.0,0.432,4.338,9.0
6.15,2.0,0.875,8.797,-4.675
2.97,2.0,0.908,9.283,3.607
5.28,2.0,0.851,8.731,-4.801
5.4,2.0,0.802,8.58,5.074
3.77,2.0,0.925,9.947,-0.454
4.77,2.0,0.913,9.879,-1.256
4.5,2.0,0.905,9.933,0.719
2.65,-0.12,0.735,8.251,-5.603
5.18,2.0,0.794,8.961,4.368
1.74,-0.199,0.838,9.959,-0.339
4.84,2.0,0.782,9.555,2.846
5.88,2.0,0.716,9.219,-3.806
4.9,2.0,0.764,9.967,0.257
6.05,2.0,0.407,5.492,8.347
6.17,2.0,0.33,4.764,8.786
3.59,2.0,0.626,9.221,-3.818
4.34,2.0,0.273,4.1,-9.117
5.46,2.0,0.397,6.439,7.641
5.18,2.0,0.516,8.436,-5.344
5.93,2.0,0.328,5.566,8.302
6.0,2.0,0.556,9.605,2.727
3.55,2.0,0.55,9.652,-2.558
5.72,2.0,0.54,9.667,2.502
3.85,2.0,0.348,6.274,-7.779
5.47,2.0,0.522,9.511,3.045
5.28,2.0,0.52,9.693,2.402
2.07,-0.168,0.526,9.844,-1.68
6.54,2.0,0.158,2.969,-9.548
5.26,2.0,0.52,9.923,1.124
4.88,2.0,0.436,9.084,4.158
4.51,2.0,0.366,7.743,6.318
4.89,2.0,0.445,9.747,2.19
4.5,2.0,0.247,5.562,-8.307
5.1,2.0,0.173,3.919,-9.198
5.46,2.0,0.078,1.844,-9.828
5.54,2.0,0.402,9.682,2.471
6.53,2.0,0.225,5.454,8.379
3.12,2.0,0.32,8.107,-5.845
4.72,2.0,0.311,7.948,6.061
3.76,2.0,0.354,9.337,-3.564
5.36,2.0,0.373,9.907,-1.308
3.97,2.0,0.288,7.75,6.313
6.14,2.0,0.173,5.153,8.568
5.6,2.0,0.323,9.69,2.448
4.76,2.0,0.33,9.989,0.324
4.88,2.0,0.25,8.306,-5.563
4.56,2.0,0.258,8.857,4.635
4.39,2.0,0.23,9.378,3.465
4.96,2.0,0.127,5.633,8.262
5.92,2.0,0.208,9.409,3.379
5.2,2.0,0.11,5.016,8.651
0.45,1.5,0.209,9.914,1.289
4.97,2.0,0.157,7.972,-6.035
5.52,2.0,0.136,8.536,-5.207
3.71,2.0,0.152,9.695,-2.448
4.36,2.0,0.088,8.163,-5.776
4.81,2.0,0.079,8.991,4.376
5.89,2.0,0.069,9.995,0.321
5.21,2.0,0.051,9.999,0.097
6.15,2.0,0.042,9.866,-1.63
5.04,2.0,0.04,9.861,-1.66
3.96,2.0,0.027,7.335,-6.796
3.72,2.0,0.012,5.838,8.119
1.9,2.0,0.015,7.078,7.065
2.65,-0.083,0.01,7.964,6.048
4.3,2.0,0.002,6.955,7.186
5.71,2.0,-0.029,6.704,7.42
4.92,2.0,-0.079,9.829,-1.839
4.12,2.0,-0.103,9.858,1.676
6.08,2.0,-0.105,7.323,6.809
5.14,2.0,-0.142,9.414,3.369
4.64,2.0,-0.161,9.387,3.443
4.16,2.0,-0.165,9.185,3.95
5.65,2.0,-0.16,8.463,-5.325
5.67,2.0,-0.216,9.953,0.945
5.63,2.0,-0.216,9.971,0.725
4.92,2.0,-0.209,9.587,-2.838
5.9,2.0,-0.161,7.314,6.818
5.72,2.0,-0.109,4.06,-9.138
4.67,2.0,-0.259,9.659,-2.577
5.35,2.0,-0.225,7.825,6.223
6.15,2.0,-0.216,7.393,-6.73
5.0,2.0,-0.261,7.955,-6.053
4.42,2.0,-0.319,9.664,2.549
5.28,2.0,-0.317,9.44,-3.283
5.5,2.0,-0.255,7.409,-6.711
5.06,2.0,-0.138,3.607,-9.326
5.75,2.0,-0.39,9.189,3.926
5.35,2.0,-0.225,5.155,8.566
5.08,2.0,-0.117,2.627,-9.648
4.72,2.0,-0.258,5.734,-8.188
5.01,2.0,-0.203,4.128,-9.106
5.83,2.0,-0.458,9.094,4.134
5.78,2.0,-0.334,6.59,7.514
4.45,2.0,-0.505,9.681,2.455
5.76,2.0,-0.493,9.396,3.386
4.95,2.0,-0.505,9.593,2.778
6.51,2.0,-0.495,9.198,3.893
5.36,2.0,-0.23,4.106,9.115
5.2,2.0,-0.612,9.426,3.282
3.99,2.0,-0.644,9.919,-1.093
3.31,2.0,-0.599,9.219,3.828
4.32,2.0,-0.584,8.684,4.924
5.34,2.0,-0.646,9.584,2.78
5.37,2.0,-0.342,4.988,8.66
5.44,2.0,-0.67,9.738,2.173
6.09,2.0,-0.65,9.114,4.063
5.04,2.0,-0.7,9.746,2.125
4.37,2.0,-0.59,8.156,-5.756
6.24,2.0,-0.678,9.129,4.026
5.39,2.0,-0.735,9.822,1.727
6.53,2.0,-0.528,6.873,7.245
6.76,2.0,-0.753,9.796,-1.861
5.01,2.0,-0.372,4.755,8.789
6.33,2.0,-0.55,6.879,7.237
6.95,2.0,-0.776,9.153,3.951
4.44,2.0,-0.44,5.13,8.573
5.27,2.0,-0.851,9.87,-1.361
3.02,2.0,-0.766,8.621,-5.01
5.34,2.0,-0.565,5.928,8.034
6.05,2.0,-0.49,5.071,8.605
3.85,2.0,-0.804,8.306,-5.51
5.56,2.0,-0.338,3.406,-9.396
1.98,-0.24,-0.941,9.466,-3.083
5.6,2.0,-0.553,5.511,-8.326
2.87,2.0,-0.924,9.192,3.829
4.39,2.0,-1.032,9.914,0.801
6.72,2.0,-1.032,9.914,0.801
-0.62,0.164,-0.632,6.027,-7.954
4.92,2.0,-0.707,6.484,7.58
5.37,2.0,-0.385,3.449,-9.378
5.76,2.0,-0.747,6.626,-7.452
5.21,2.0,-0.611,5.201,8.519
6.22,2.0,-1.139,9.297,3.501
5.06,2.0,-1.213,9.891,-0.83
4.47,2.0,-1.033,8.363,-5.385
5.75,2.0,-1.071,8.55,5.074
3.76,2.0,-1.245,9.846,-1.224
4.13,2.0,-1.183,9.309,3.455
5.88,2.0,-0.897,6.801,7.276
5.86,2.0,-0.706,5.227,8.496
6.38,2.0,-1.323,9.526,2.74
4.34,2.0,-1.271,9.088,-3.974
5.69,2.0,-0.721,5.137,-8.549
5.88,2.0,-1.401,9.865,0.847
4.47,2.0,-1.419,9.816,1.276
6.45,2.0,-0.854,5.621,8.227
6.44,2.0,-1.498,9.799,1.318
4.54,2.0,-1.403,9.1,-3.902
5.26,2.0,-1.351,8.724,4.698
5.71,2.0,-1.498,9.355,-3.2
3.95,2.0,-1.505,9.32,-3.298
5.87,2.0,-0.894,5.394,8.373
5.94,2.0,-0.782,4.71,8.787
1.93,0.001,-1.571,9.464,2.823
3.17,2.0,-1.196,7.191,-6.845
4.42,2.0,-1.563,9.368,-3.13
5.76,2.0,-1.458,8.625,4.846
5.7,2.0,-1.297,7.619,6.346
5.34,2.0,-1.293,7.562,6.415
4.8,2.0,-1.259,7.266,6.754
6.02,2.0,-1.513,8.677,4.735
5.61,2.0,-0.278,1.572,-9.872
4.66,2.0,-1.752,9.694,1.719
6.45,2.0,-1.581,8.67,4.725
5.2,2.0,-1.753,9.367,3.031
5.04,2.0,-1.332,7.004,7.012
3.06,2.0,-1.725,8.888,4.247
4.49,2.0,-1.857,9.556,2.288
5.23,2.0,-1.652,8.405,-5.161
5.42,2.0,-1.698,8.582,4.844
-1.44,0.009,-1.875,9.392,-2.876
3.35,2.0,-1.914,9.558,2.232
6.47,2.0,-1.941,9.561,2.197
4.86,2.0,-1.019,4.981,8.611
5.92,2.0,-1.994,9.685,1.493
5.24,2.0,-1.467,7.094,6.893
5.34,2.0,-1.1,5.309,8.403
5.28,2.0,-1.966,9.483,-2.491
6.07,2.0,-1.902,9.139,-3.586
4.77,2.0,-2.03,9.691,1.398
5.27,2.0,-1.618,7.72,-6.147
5.22,2.0,-1.36,6.446,7.523
4.48,2.0,-2.071,9.774,0.421
3.24,2.0,-0.982,4.6,-8.825
5.87,2.0,-2.072,9.377,2.79
3.5,2.0,-1.82,8.234,-5.374
2.94,2.0,-1.372,6.195,-7.729
5.68,2.0,-2.123,9.493,2.32
4.99,2.0,-1.638,7.275,6.663
5.14,2.0,-0.841,3.721,9.244
5.41,2.0,-0.726,3.18,-9.453
5.28,2.0,-2.071,9.053,3.707
3.6,2.0,-1.894,8.075,5.586
6.1,2.0,-1.785,7.578,6.275
5.34,2.0,-1.167,4.947,8.612
6.32,2.0,-1.803,7.623,6.217
4.82,2.0,-2.173,9.128,-3.457
5.11,2.0,-0.836,3.503,9.329
6.46,2.0,-1.825,7.61,6.226
3.89,2.0,-2.135,8.869,-4.097
4.08,2.0,-2.291,9.508,-2.086
4.73,2.0,-2.3,9.461,2.28
5.8,2.0,-2.23,9.103,-3.487
5.74,2.0,-2.159,8.773,4.286
4.66,2.0,-2.256,9.114,-3.443
5.0,2.0,-2.351,9.412,-2.427
4.36,2.0,-2.318,9.275,-2.933
5.85,2.0,-1.688,6.703,7.227
5.45,2.0,-0.449,1.78,-9.83
4.35,2.0,-1.295,5.074,8.519
4.9,2.0,-1.756,6.838,7.083
5.07,2.0,-2.088,8.012,-5.608
1.5,-0.211,-2.214,8.464,-4.844
6.2,2.0,-2.279,8.688,4.396
6.02,2.0,-2.006,7.615,6.164
6.4,2.0,-2.308,8.693,4.37
5.73,2.0,-2.498,9.279,2.77
3.49,2.0,-2.351,8.516,-4.685
5.2,2.0,-2.453,8.784,4.102
4.99,2.0,-2.703,9.599,-0.739
3.02,2.0,-2.484,8.804,-4.041
4.11,2.0,-2.645,9.26,-2.695
4.92,2.0,-1.784,6.233,-7.614
5.2,2.0,-2.039,7.105,-6.735
4.01,2.0,-2.585,8.999,3.514
6.0,2.0,-2.595,8.857,3.849
5.47,2.0,-2.826,9.191,2.745
1.83,0.671,-2.634,8.562,-4.445
5.68,2.0,-0.986,3.19,-9.426
3.78,2.0,-0.986,3.189,-9.426
4.83,2.0,-2.278,7.354,-6.382
4.91,2.0,-3.008,9.508,-0.739
4.41,2.0,-2.639,8.226,5.037
5.75,2.0,-2.734,8.492,4.518
5.44,2.0,-3.065,9.519,-0.053
4.91,2.0,-2.38,7.361,6.337
4.15,2.0,-3.084,9.512,-0.086
5.91,2.0,-2.786,8.55,-4.375
5.3,2.0,-2.358,7.229,-6.494
5.85,2.0,-2.837,8.674,4.088
4.49,2.0,-2.133,6.51,-7.285
7.05,2.0,-2.814,8.556,4.344
4.87,2.0,-2.214,6.692,-7.094
5.07,2.0,-3.023,9.117,2.783
4.42,2.0,-2.244,6.752,-7.027
4.42,2.0,-2.853,8.495,-4.439
5.84,2.0,-2.905,8.594,4.208
4.01,2.0,-2.863,8.457,-4.505
6.42,2.0,-3.228,9.465,-0.028
5.2,2.0,-1.644,4.78,8.628
5.75,2.0,-2.876,8.357,4.679
4.83,2.0,-3.013,8.675,-3.958
3.97,2.0,-1.235,3.544,-9.269
2.71,2.0,-2.634,7.529,-6.032
5.87,2.0,-2.525,7.126,6.545
3.58,2.0,-3.204,9.035,2.847
5.24,2.0,-2.604,7.298,-6.322
5.11,2.0,-2.696,7.546,-5.982
4.88,2.0,-3.061,8.565,-4.156
4.37,2.0,-3.053,8.537,-4.219
3.5,2.0,-3.176,8.712,3.743
5.09,2.0,-3.28,8.778,3.493
5.12,2.0,-2.807,7.504,5.985
6.36,2.0,-1.762,4.695,8.652
6.86,2.0,-2.014,5.327,8.22
5.8,2.0,-2.015,5.327,8.22
6.17,2.0,-3.28,8.605,3.899
5.04,2.0,-3.227,8.465,4.234
2.45,-0.083,-3.129,8.14,-4.894
5.23,2.0,-2.722,7.079,6.517
5.77,2.0,-3.195,8.263,4.639
5.37,2.0,-3.548,9.128,2.023
5.54,2.0,-0.691,1.761,-9.819
4.99,2.0,-3.603,9.188,1.612
3.78,2.0,-3.232,8.234,4.664
4.61,2.0,-2.413,6.071,7.571
5.94,2.0,-3.474,8.715,3.462
2.89,2.0,-3.673,9.189,1.442
5.2,2.0,-3.477,8.634,3.656
5.22,2.0,-3.72,9.203,1.209
4.33,2.0,-3.707,9.157,1.552
5.41,2.0,-2.953,7.211,-6.268
4.16,2.0,-3.222,7.866,5.267
3.25,2.0,-2.762,6.733,-6.858
5.07,2.0,-3.352,8.158,4.713
4.55,2.0,-3.735,9.04,2.08
5.01,2.0,-3.375,8.166,4.682
5.35,2.0,-2.475,5.98,7.624
5.24,2.0,-3.909,9.198,0.334
5.59,2.0,-3.949,9.169,0.574
5.27,2.0,-3.821,8.829,2.727
5.42,2.0,-3.207,7.389,-5.926
5.83,2.0,-3.991,9.15,0.588
1.58,0.034,-3.406,7.778,5.283
4.06,2.0,-3.625,8.148,4.524
5.14,2.0,-4.108,9.089,-0.717
4.53,2.0,-3.378,7.466,-5.731
4.69,2.0,-3.758,8.217,-4.284
5.58,2.0,-3.413,7.441,5.743
4.89,2.0,-3.452,7.474,5.676
0.4,0.432,-4.181,9.038,0.911
4.84,2.0,-3.299,7.12,-6.199
5.04,2.0,-4.007,8.644,3.036
5.73,2.0,-3.316,7.132,-6.176
5.76,2.0,-3.312,7.119,-6.192
6.09,2.0,-2.314,4.917,8.395
3.94,2.0,-4.216,8.915,-1.659
3.93,2.0,-1.285,2.699,-9.543
4.93,2.0,-2.257,4.678,8.546
4.23,2.0,-3.815,7.881,4.83
4.63,2.0,-3.84,7.913,-4.758
3.94,2.0,-3.829,7.868,-4.841
5.3,2.0,-3.951,8.091,4.35
3.57,2.0,-4.008,8.177,4.131
6.53,2.0,-4.153,8.408,3.472
3.62,2.0,-3.495,7.067,-6.152
1.16,0.991,-3.915,7.911,4.699
7.03,2.0,-4.297,8.667,-2.535
6.06,2.0,-4.297,8.666,-2.536
5.03,2.0,-4.317,8.663,-2.515
4.89,2.0,-4.236,8.484,3.175
5.25,2.0,-4.394,8.787,1.868
6.5,2.0,-2.046,4.082,8.897
5.93,2.0,-1.864,3.712,9.096
5.14,2.0,-3.773,7.445,5.507
5.48,2.0,-4.435,8.71,-2.112
4.4,2.0,-4.086,8.011,-4.374
4.69,2.0,-3.101,6.063,-7.323
6.18,2.0,-4.195,8.183,3.93
4.1,2.0,-3.166,6.131,-7.238
3.34,2.0,-4.165,8.061,-4.204
5.17,2.0,-4.399,8.478,-2.962
6.37,2.0,-4.566,8.616,-2.219
5.12,2.0,-4.681,8.831,0.308
5.16,2.0,-4.549,8.576,-2.402
3.71,2.0,-3.572,6.703,-6.505
5.69,2.0,-4.55,8.528,-2.562
4.49,2.0,-3.675,6.865,-6.275
4.22,2.0,-3.169,5.878,-7.443
4.97,2.0,-4.243,7.856,4.503
6.25,2.0,-3.243,5.943,7.359
5.47,2.0,-3.238,5.92,7.38
5.38,2.0,-4.547,8.231,3.401
3.46,2.0,-2.937,5.256,-7.985
4.2,2.0,-4.497,8.041,-3.888
5.8,2.0,-4.702,8.396,2.721
5.08,2.0,-3.517,6.26,-6.96
5.14,2.0,-3.443,6.094,-7.142
5.3,2.0,-4.934,8.69,0.388
5.09,2.0,-4.56,7.971,-3.957
4.93,2.0,-4.98,8.648,-0.642
5.6,2.0,-4.802,8.252,2.975
5.87,2.0,-4.549,7.805,4.288
4.69,2.0,-5.045,8.631,-0.243
5.99,2.0,-4.85,8.274,2.833
6.02,2.0,-2.507,4.269,8.688
6.32,2.0,-4.587,7.809,4.24
6.48,2.0,-2.756,4.651,8.413
4.94,2.0,-4.54,7.592,4.663
2.21,-0.269,-3.933,6.573,-6.428
6.12,2.0,-4.872,8.054,-3.376
5.14,2.0,-5.055,8.324,2.27
5.96,2.0,-4.833,7.863,3.849
6.35,2.0,-3.837,6.19,6.853
2.83,2.0,-4.814,7.738,-4.116
5.3,2.0,-4.92,7.891,3.678
4.35,2.0,-1.931,3.093,-9.312
4.78,2.0,-3.309,5.272,7.827
4.36,2.0,-5.314,8.455,-0.521
6.25,2.0,-5.175,8.225,2.358
4.4,2.0,-5.039,7.984,-3.296
1.75,-0.145,-3.63,5.723,-7.353
5.73,2.0,-4.864,7.602,4.306
5.53,2.0,-5.242,8.175,-2.385
4.72,2.0,-5.282,8.191,-2.237
4.44,2.0,-4.177,6.472,-6.377
4.67,2.0,-5.197,7.987,3.032
5.62,2.0,-4.769,7.264,4.948
4.99,2.0,-5.288,8.04,-2.721
4.78,2.0,-4.456,6.765,-5.864
4.42,2.0,-4.208,6.354,-6.474
3.53,2.0,-5.539,8.172,1.596
6.32,2.0,-5.416,7.925,-2.804
5.63,2.0,-2.866,4.176,8.623
4.05,2.0,-1.286,1.862,-9.741
4.44,2.0,-4.56,6.6,-5.971
5.33,2.0,-1.816,2.599,-9.484
5.63,2.0,-1.82,2.598,-9.484
5.13,2.0,-5.103,7.283,4.574
5.89,2.0,-3.07,4.366,8.457
5.92,2.0,-5.256,7.47,4.071
4.34,2.0,-1.248,1.772,-9.762
4.83,2.0,-4.849,6.837,-5.454
5.28,2.0,-1.66,2.325,-9.583
1.86,1.196,-2.956,4.124,-8.617
4.79,2.0,-3.861,5.387,-7.488
6.13,2.0,-5.684,7.911,-2.259
4.25,2.0,-4.256,5.92,6.844
5.94,2.0,-5.558,7.695,3.145
6.11,2.0,-5.774,7.953,1.845
5.61,2.0,-5.886,8.058,-0.654
3.91,2.0,-5.923,8.029,-0.681
3.77,2.0,-2.403,3.255,-9.145
6.11,2.0,-5.681,7.69,2.932
5.6,2.0,-5.95,8.007,-0.695
5.58,2.0,-5.271,7.094,4.678
6.51,2.0,-5.432,7.297,4.152
5.56,2.0,-5.829,7.825,2.191
6.3,2.0,-5.327,7.148,4.53
5.08,2.0,-3.606,4.803,-7.996
6.05,2.0,-5.511,7.273,4.09
5.94,2.0,-5.855,7.726,2.455
3.35,2.0,-2.982,3.877,8.722
5.71,2.0,-5.605,7.206,4.08
5.33,2.0,-5.839,7.501,3.106
6.48,2.0,-6.084,7.741,1.748
5.33,2.0,-5.792,7.366,3.492
6.35,2.0,-5.652,7.169,4.081
6.2,2.0,-4.991,6.31,5.939
5.01,2.0,-4.022,5.024,-7.654
5.76,2.0,-5.029,6.282,5.937
5.27,2.0,-3.301,4.103,-8.501
6.55,2.0,-5.907,7.339,3.353
4.84,2.0,-3.322,4.127,-8.481
5.72,2.0,-6.214,7.711,-1.389
5.92,2.0,-6.241,7.632,1.677
4.11,2.0,-4.644,5.652,-6.819
4.14,2.0,-6.318,7.688,0.994
5.24,2.0,-5.701,6.923,-4.424
6.54,2.0,-6.275,7.606,1.663
4.45,2.0,-6.375,7.682,0.583
5.19,2.0,-2.147,2.58,-9.42
5.63,2.0,-2.703,3.244,9.065
4.86,2.0,-5.583,6.67,-4.933
6.65,2.0,-6.04,7.216,3.384
4.98,2.0,-6.277,7.479,-2.16
3.97,2.0,-5.248,6.249,-5.78
6.39,2.0,-6.043,7.195,3.421
6.61,2.0,-6.047,7.195,3.415
4.59,2.0,-2.788,3.316,9.013
3.6,2.0,-3.881,4.614,-7.978
6.29,2.0,-6.072,7.207,3.345
4.31,2.0,-3.248,3.849,-8.639
3.77,2.0,-4.427,5.247,-7.272
5.35,2.0,-4.502,5.317,7.173
4.74,2.0,-4.385,5.17,-7.351
5.46,2.0,-1.239,1.459,-9.815
4.87,2.0,-6.236,7.319,-2.747
5.62,2.0,-6.375,7.385,2.195
4.3,2.0,-6.524,7.556,0.593
4.66,2.0,-6.084,7.042,3.66
3.68,2.0,-5.479,6.326,-5.474
5.15,2.0,-4.226,4.875,-7.641
4.63,2.0,-6.498,7.496,-1.259
4.05,2.0,-4.835,5.543,-6.775
3.94,2.0,-6.255,7.153,3.116
1.93,0.043,-3.804,4.349,-8.162
5.63,2.0,-6.484,7.409,1.751
6.13,2.0,-6.562,7.481,0.99
6.12,2.0,-5.68,6.456,5.105
3.87,2.0,-4.6,5.198,-7.198
4.32,2.0,-6.454,7.271,-2.343
6.58,2.0,-5.828,6.548,4.812
4.03,2.0,-5.829,6.548,4.811
4.5,2.0,-3.644,4.093,-8.365
3.38,2.0,-6.61,7.42,1.118
5.89,2.0,-6.509,7.296,2.098
4.35,2.0,-6.671,7.38,1.017
5.3,2.0,-6.723,7.378,-0.601
4.94,2.0,-4.747,5.19,-7.109
4.02,2.0,-5.996,6.513,-4.65
5.09,2.0,-4.66,5.061,-7.257
6.36,2.0,-6.547,7.081,2.647
5.55,2.0,-6.753,7.269,-1.249
5.15,2.0,-4.927,5.286,6.912
6.25,2.0,-6.021,6.429,4.735
5.67,2.0,-5.769,6.155,5.369
5.96,2.0,-6.02,6.422,4.746
5.72,2.0,-3.226,3.418,8.826
5.4,2.0,-5.933,6.239,5.087
3.84,2.0,-3.391,3.54,-8.716
3.11,2.0,-6.89,7.173,1.036
6.07,2.0,-6.864,7.139,-1.387
4.87,2.0,-6.138,6.383,-4.646
5.23,2.0,-6.128,6.365,4.684
5.44,2.0,-6.802,7.048,2.015
4.68,2.0,-4.213,4.351,-7.957
5.43,2.0,-0.527,0.542,-9.971
5.44,2.0,-5.857,6.015,5.433
4.93,2.0,-3.569,3.665,-8.592
5.22,2.0,-6.737,6.901,2.643
5.68,2.0,-6.739,6.883,2.686
6.25,2.0,-6.055,6.163,5.035
4.26,2.0,-6.874,6.966,2.055
3.12,2.0,-4.711,4.744,7.436
5.17,2.0,-3.624,3.642,-8.579
5.23,2.0,-5.957,5.981,5.361
4.45,2.0,-5.318,5.314,-6.594
5.89,2.0,-6.017,5.943,5.336
6.07,2.0,-6.298,6.199,4.68
4.0,2.0,-2.861,2.801,-9.163
4.74,2.0,-2.721,2.661,9.247
5.45,2.0,-6.513,6.359,4.139
3.57,2.0,-4.884,4.732,7.332
3.75,2.0,-4.9,4.726,-7.325
6.67,2.0,-6.362,6.135,4.679
4.99,2.0,-7.224,6.857,0.888
5.23,2.0,-7.18,6.71,1.851
2.23,1.665,-5.311,4.953,-6.875
5.42,2.0,-6.356,5.927,4.948
4.62,2.0,-6.583,6.136,-4.362
5.15,2.0,-2.877,2.674,9.196
5.6,2.0,-7.252,6.721,-1.494
5.95,2.0,-6.559,6.074,4.482
4.46,2.0,-4.558,4.218,7.837
5.16,2.0,-6.816,6.281,3.753
5.47,2.0,-7.274,6.69,-1.528
5.59,2.0,-6.36,5.831,-5.055
6.04,2.0,-6.846,6.254,3.745
4.8,2.0,-2.869,2.62,9.214
4.67,2.0,-3.3,3.0,8.95
3.43,2.0,-3.816,3.467,-8.569
3.96,2.0,-3.443,3.119,-8.855
6.49,2.0,-7.187,6.454,2.587
6.1,2.0,-7.387,6.626,-1.238
1.67,0.07,-2.588,2.306,-9.38
5.3,2.0,-5.454,4.834,6.848
5.18,2.0,-3.587,3.164,8.782
3.89,2.0,-7.494,6.609,0.404
5.24,2.0,-5.466,4.818,-6.849
5.36,2.0,-7.271,6.363,2.578
4.92,2.0,-5.892,5.139,-6.235
4.63,2.0,-5.989,5.217,-6.076
4.8,2.0,-4.437,3.851,8.092
4.34,2.0,-4.054,3.517,-8.438
5.49,2.0,-7.479,6.462,-1.52
5.24,2.0,-7.52,6.498,-1.107
2.21,0.189,-3.872,3.334,-8.596
5.26,2.0,-4.782,4.081,-7.777
3.82,2.0,-6.108,5.178,5.991
4.77,2.0,-7.488,6.295,-2.075
4.8,2.0,-7.568,6.323,-1.66
4.79,2.0,-3.561,2.963,-8.862
3.14,2.0,-6.346,5.275,5.649
4.71,2.0,-6.925,5.734,-4.378
2.47,-0.141,-4.426,3.645,-8.193
4.71,2.0,-6.789,5.537,-4.823
5.34,2.0,-1.245,1.007,-9.871
4.47,2.0,-6.99,5.628,4.412
5.6,2.0,-7.779,6.221,-0.892
5.09,2.0,-4.673,3.707,-8.026
6.53,2.0,-7.758,6.103,-1.603
4.72,2.0,-7.271,5.716,-3.802
1.99,1.44,-7.779,6.1,-1.505
5.4,2.0,-7.794,6.064,1.574
5.72,2.0,-7.813,6.077,1.424
4.59,2.0,-7.902,6.11,-0.483
4.51,2.0,-6.406,4.949,-5.871
3.6,2.0,-6.05,4.613,-6.49
5.87,2.0,-6.62,5.046,5.542
5.75,2.0,-6.753,5.146,-5.283
3.16,2.0,-4.335,3.289,-8.39
3.65,2.0,-3.612,2.734,8.915
5.91,2.0,-6.772,5.125,-5.28
5.39,2.0,-6.524,4.937,5.751
4.32,2.0,-7.346,5.55,3.902
4.99,2.0,-7.83,5.903,1.959
5.07,2.0,-7.871,5.933,1.688
4.54,2.0,-7.984,6.017,-0.207
3.17,2.0,-4.966,3.713,7.845
6.19,2.0,-6.453,4.796,5.946
5.07,2.0,-1.265,0.937,-9.875
5.01,2.0,-5.033,3.719,-7.799
4.54,2.0,-6.475,4.782,5.934
4.08,2.0,-4.118,3.035,-8.592
4.54,2.0,-2.776,2.045,9.387
5.56,2.0,-8.01,5.898,-1.03
5.4,2.0,-6.554,4.776,5.851
6.32,2.0,-7.834,5.698,2.483
4.34,2.0,-5.283,3.81,-7.588
5.73,2.0,-7.785,5.602,2.83
5.0,2.0,-8.063,5.794,1.19
6.4,2.0,-8.028,5.734,-1.637
5.51,2.0,-5.936,4.233,-6.844
5.28,2.0,-6.222,4.423,6.46
4.68,2.0,-8.126,5.772,0.811
4.51,2.0,-3.922,2.763,-8.774
6.31,2.0,-8.047,5.646,-1.834
3.9,2.0,-8.186,5.74,-0.199
5.07,2.0,-7.944,5.547,-2.475
3.52,2.0,-8.098,5.61,1.718
4.76,2.0,-7.536,5.214,-4.002
6.26,2.0,-7.405,5.106,4.37
5.61,2.0,-6.336,4.354,6.395
6.12,2.0,-6.757,4.613,5.749
5.64,2.0,-7.172,4.858,4.996
5.56,2.0,-4.881,3.302,-8.079
5.36,2.0,-8.037,5.435,2.423
4.78,2.0,-7.34,4.941,-4.659
3.69,2.0,-3.841,2.561,-8.871
2.97,2.0,-7.628,5.056,4.031
5.43,2.0,-1.909,1.26,-9.735
5.67,2.0,-8.171,5.389,2.047
2.92,2.0,-3.526,2.309,-9.068
6.44,2.0,-8.203,5.356,2.005
6.1,2.0,-7.861,5.016,3.613
5.09,2.0,-5.886,3.752,-7.161
6.24,2.0,-8.421,5.339,0.757
3.78,2.0,-4.351,2.745,8.575
6.45,2.0,-8.239,5.197,2.261
6.01,2.0,-8.44,5.313,-0.74
4.11,2.0,-8.186,5.141,-2.562
4.58,2.0,-5.827,3.653,-7.259
5.29,2.0,-7.721,4.83,4.13
4.55,2.0,-4.978,3.108,8.097
6.02,2.0,-8.477,5.287,0.428
5.07,2.0,-8.407,5.228,-1.41
3.88,2.0,-7.638,4.737,4.385
3.52,2.0,-4.981,2.967,-8.148
5.11,2.0,-6.492,3.836,6.568
5.26,2.0,-8.419,4.948,2.155
5.23,2.0,-6.996,4.085,-5.862
6.69,2.0,-8.639,5.002,0.59
4.68,2.0,-8.58,4.943,1.399
5.53,2.0,-1.175,0.674,-9.908
5.37,2.0,-7.369,4.211,5.288
6.43,2.0,-8.736,4.835,0.558
4.6,2.0,-8.543,4.681,-2.261
6.2,2.0,-8.762,4.719,0.978
3.48,2.0,-8.441,4.52,2.884
4.49,2.0,-7.201,3.852,5.771
4.39,2.0,-8.693,4.627,1.736
4.48,2.0,-8.828,4.697,-0.065
1.36,-0.087,-8.645,4.579,2.074
4.85,2.0,-5.471,2.88,-7.86
5.91,2.0,-8.777,4.563,-1.462
3.61,2.0,-8.676,4.488,-2.139
5.64,2.0,-8.793,4.532,-1.464
6.43,2.0,-8.662,4.431,2.31
5.77,2.0,-8.897,4.494,0.805
3.29,2.0,-3.054,1.527,-9.399
3.85,2.0,-6.649,3.288,-6.707
6.47,2.0,-7.651,3.769,5.22
5.49,2.0,-7.841,3.814,4.895
6.46,2.0,-7.893,3.83,4.8
5.95,2.0,-8.252,4.001,3.988
5.42,2.0,-8.744,4.233,2.373
3.43,2.0,-8.26,3.998,3.974
3.39,2.0,-4.322,2.083,-8.774
3.45,2.0,-6.598,3.179,6.809
5.81,2.0,-8.289,3.987,3.924
5.25,2.0,-8.93,4.277,-1.404
4.78,2.0,-8.54,3.994,3.333
2.01,1.128,-8.524,3.976,3.394
4.5,2.0,-5.072,2.34,-8.295
6.66,2.0,-9.075,4.182,0.4
6.16,2.0,-8.788,4.012,2.584
4.82,2.0,-6.804,3.089,-6.646
3.06,2.0,-6.82,3.096,6.626
6.06,2.0,-9.058,4.08,1.139
5.89,2.0,-7.569,3.405,5.579
5.93,2.0,-9.102,4.079,-0.71
5.52,2.0,-7.601,3.378,5.551
5.61,2.0,-9.05,3.97,1.527
4.72,2.0,-7.62,3.317,5.562
3.83,2.0,-8.779,3.813,-2.896
4.28,2.0,-7.872,3.376,-5.161
6.01,2.0,-9.066,3.865,1.696
3.81,2.0,-4.776,2.03,-8.548
4.2,2.0,-7.378,3.136,5.977
5.19,2.0,-9.22,3.843,-0.478
5.57,2.0,-7.946,3.308,-5.091
5.79,2.0,-7.189,2.974,6.283
5.08,2.0,-9.243,3.815,-0.111
5.9,2.0,-7.828,3.169,5.355
3.3,2.0,-4.398,1.776,-8.804
5.43,2.0,-8.994,3.625,2.442
3.84,2.0,-9.163,3.664,1.617
5.02,2.0,-6.333,2.528,-7.314
5.57,2.0,-7.616,3.019,5.734
5.08,2.0,-8.517,3.354,-4.027
5.07,2.0,-9.248,3.606,1.211
5.67,2.0,-9.215,3.582,1.504
4.11,2.0,-1.842,0.712,-9.803
4.45,2.0,-5.005,1.931,-8.439
6.01,2.0,-8.963,3.428,-2.814
5.08,2.0,-4.732,1.808,-8.622
6.29,2.0,-7.526,2.875,5.924
3.84,2.0,-6.233,2.352,-7.458
6.42,2.0,-7.757,2.905,5.603
4.91,2.0,-8.972,3.328,-2.903
4.68,2.0,-7.955,2.946,5.296
5.84,2.0,-7.404,2.727,6.144
4.29,2.0,-5.303,1.948,-8.251
6.25,2.0,-9.413,3.361,-0.304
2.74,2.0,-4.08,1.426,-9.018
5.51,2.0,-8.462,2.954,4.435
5.77,2.0,-9.413,3.271,0.828
5.08,2.0,-8.684,3.015,3.938
4.58,2.0,-4.643,1.609,-8.709
6.27,2.0,-9.463,3.205,0.434
5.46,2.0,-1.568,0.53,-9.862
4.45,2.0,-1.558,0.523,-9.864
5.36,2.0,-8.154,2.734,5.103
5.5,2.0,-8.978,2.986,3.238
5.49,2.0,-9.199,3.059,2.452
2.69,2.0,-6.176,2.043,-7.595
5.44,2.0,-9.066,2.995,-2.973
6.15,2.0,-8.296,2.657,4.911
5.32,2.0,-9.366,2.987,1.83
6.61,2.0,-9.504,3.029,-0.702
3.11,2.0,-9.154,2.903,-2.789
6.05,2.0,-8.422,2.66,4.691
5.8,2.0,-9.426,2.958,-1.547
3.79,2.0,-7.922,2.372,5.623
5.23,2.0,-8.996,2.686,-3.443
3.78,2.0,-4.956,1.48,-8.558
5.45,2.0,-9.578,2.85,-0.372
4.66,2.0,-6.99,2.071,6.844
6.19,2.0,-8.663,2.537,4.304
4.3,2.0,-8.725,2.518,4.187
5.91,2.0,-9.608,2.768,0.129
5.91,2.0,-9.557,2.739,1.077
6.36,2.0,-8.682,2.466,4.305
4.6,2.0,-7.67,2.173,-6.037
6.19,2.0,-0.909,0.247,-9.956
4.08,2.0,-9.168,2.466,-3.14
4.37,2.0,-7.154,1.912,-6.721
4.84,2.0,-9.646,2.559,0.631
4.98,2.0,-9.613,2.542,1.063
4.73,2.0,-9.67,2.509,-0.433
2.34,0.033,-5.359,1.39,8.328
4.42,2.0,-9.091,2.337,3.45
5.95,2.0,-9.699,2.436,-0.0
1.81,1.061,-4.591,1.15,8.809
4.62,2.0,-9.634,2.357,1.277
4.92,2.0,-8.635,2.1,-4.586
5.69,2.0,-8.641,2.076,-4.585
4.62,2.0,-4.504,1.07,-8.864
5.52,2.0,-9.727,2.295,0.341
6.48,2.0,-8.944,2.079,3.959
3.93,2.0,-5.025,1.146,-8.569
5.7,2.0,-8.862,2.013,4.172
3.0,2.0,-6.961,1.554,7.009
4.46,2.0,-9.013,1.93,-3.879
5.4,2.0,-9.797,2.004,-0.012
2.56,0.128,-9.178,1.863,3.506
3.33,2.0,-9.448,1.912,2.661
4.56,2.0,-9.023,1.787,3.923
5.31,2.0,-9.552,1.862,2.302
4.45,2.0,-9.802,1.876,-0.637
5.18,2.0,-9.821,1.852,0.351
3.79,2.0,-8.382,1.546,5.23
3.49,2.0,-8.241,1.509,5.46
5.9,2.0,-9.836,1.782,0.288
3.56,2.0,-9.517,1.706,-2.551
3.9,2.0,-5.724,0.983,-8.14
4.05,2.0,-9.802,1.678,1.05
5.08,2.0,-9.347,1.507,-3.219
4.0,2.0,-9.71,1.541,1.827
5.39,2.0,-9.874,1.562,0.246
4.81,2.0,-9.704,1.511,-1.884
4.06,2.0,-9.416,1.454,-3.038
5.58,2.0,-9.483,1.434,2.833
6.35,2.0,-9.867,1.482,0.673
6.49,2.0,-9.881,1.443,0.526
5.93,2.0,-9.668,1.395,-2.14
4.95,2.0,-9.89,1.393,0.498
5.74,2.0,-9.556,1.271,2.658
4.77,2.0,-9.903,1.29,-0.524
5.54,2.0,-9.41,1.219,3.158
3.82,2.0,-3.502,0.439,9.356
6.27,2.0,-9.614,1.191,2.481
5.07,2.0,-5.046,0.625,-8.611
5.12,2.0,-5.035,0.622,-8.618
4.93,2.0,-8.66,1.053,-4.888
3.54,2.0,-8.435,0.998,-5.278
5.76,2.0,-9.923,1.114,0.534
5.95,2.0,-9.515,1.054,2.89
4.62,2.0,-5.805,0.642,-8.117
5.64,2.0,-6.733,0.731,-7.358
3.11,2.0,-4.512,0.479,-8.912
5.26,2.0,-6.701,0.706,-7.389
5.19,2.0,-3.512,0.368,9.356
4.7,2.0,-9.803,1.001,-1.702
4.3,2.0,-9.948,1.004,-0.144
5.64,2.0,-2.425,0.241,-9.699
5.46,2.0,-6.692,0.657,-7.402
5.24,2.0,-9.856,0.929,1.415
5.48,2.0,-9.694,0.905,-2.284
4.7,2.0,-8.186,0.709,-5.699
5.26,2.0,-9.281,0.78,3.641
5.32,2.0,-3.937,0.302,9.188
4.71,2.0,-9.471,0.631,-3.148
4.84,2.0,-9.876,0.635,1.436
3.63,2.0,-3.943,0.248,-9.186
4.04,2.0,-9.916,0.613,1.137
3.69,2.0,-6.707,0.409,7.406
5.31,2.0,-9.883,0.522,1.434
4.5,2.0,-9.371,0.492,3.456
4.75,2.0,-3.932,0.202,-9.192
2.14,0.09,-9.667,0.462,2.516
4.3,2.0,-4.412,0.199,-8.972
3.59,2.0,-9.987,0.406,0.308
4.47,2.0,-7.044,0.272,-7.092
4.29,2.0,-8.295,0.257,-5.579
2.41,0.044,-5.919,0.159,8.059
5.58,2.0,-9.889,0.214,1.468
5.53,2.0,-9.628,0.182,2.697
5.17,2.0,-9.554,0.166,-2.949
4.88,2.0,-2.041,0.003,-9.789
5.36,2.0,-9.98,0.002,0.638
5.28,2.0,-9.417,-0.035,-3.364
4.65,2.0,-9.933,-0.038,1.152
4.32,2.0,-4.491,-0.059,-8.935
5.89,2.0,-9.305,-0.174,3.658
4.72,2.0,-4.513,-0.085,-8.923
5.04,2.0,-2.331,-0.049,-9.724
4.12,2.0,-9.882,-0.225,1.518
4.14,2.0,-4.285,-0.129,-9.034
5.34,2.0,-6.597,-0.237,-7.512
2.58,2.0,-6.327,-0.231,-7.741
4.02,2.0,-9.077,-0.333,-4.183
5.95,2.0,-9.986,-0.422,0.331
5.72,2.0,-9.939,-0.436,1.012
3.02,2.0,-9.222,-0.408,-3.846
6.4,2.0,-9.563,-0.44,2.892
5.45,2.0,-9.153,-0.442,-4.004
3.97,2.0,-6.098,-0.31,-7.92
5.66,2.0,-8.986,-0.465,4.363
5.6,2.0,-9.351,-0.496,3.509
5.85,2.0,-9.823,-0.576,1.782
5.31,2.0,-6.968,-0.428,-7.16
2.79,2.0,-5.177,-0.343,-8.549
3.32,2.0,-5.429,-0.366,8.39
2.58,-0.107,-9.512,-0.657,-3.014
5.09,2.0,-9.64,-0.674,2.571
5.69,2.0,-7.567,-0.533,6.516
4.93,2.0,-9.116,-0.651,4.059
4.06,2.0,-3.741,-0.288,-9.269
4.24,2.0,-1.849,-0.148,-9.827
4.06,2.0,-4.369,-0.352,-8.988
5.9,2.0,-9.966,-0.814,-0.137
5.01,2.0,-5.696,-0.473,-8.206
6.26,2.0,-9.17,-0.775,3.913
6.38,2.0,-8.785,-0.749,4.719
5.28,2.0,-6.538,-0.567,7.545
3.89,2.0,-9.962,-0.867,-0.116
4.97,2.0,-9.944,-0.885,0.578
5.2,2.0,-9.22,-0.829,-3.781
4.72,2.0,-9.483,-0.86,3.056
3.59,2.0,-4.918,-0.46,-8.695
5.15,2.0,-3.805,-0.368,-9.24
5.73,2.0,-3.679,-0.357,-9.292
4.78,2.0,-8.956,-0.882,4.36
6.46,2.0,-9.909,-0.978,0.925
5.66,2.0,-9.028,-0.923,-4.201
5.32,2.0,-8.107,-0.837,-5.795
6.03,2.0,-7.328,-0.763,6.761
4.76,2.0,-6.183,-0.65,7.833
5.17,2.0,-8.93,-0.951,4.399
5.71,2.0,-8.123,-0.903,-5.762
5.01,2.0,-7.72,-0.874,6.296
4.92,2.0,-8.83,-1.022,4.582
4.82,2.0,-6.19,-0.72,-7.821
0.77,-0.243,-4.494,-0.524,-8.918
4.35,2.0,-8.747,-1.033,4.736
4.98,2.0,-8.862,-1.048,4.513
3.91,2.0,-6.349,-0.781,-7.686
5.45,2.0,-7.707,-0.959,-6.299
5.29,2.0,-8.923,-1.132,4.37
5.47,2.0,-9.052,-1.17,4.085
5.68,2.0,-9.264,-1.208,3.567
2.94,2.0,-9.506,-1.246,-2.843
6.21,2.0,-6.167,-0.813,7.83
5.01,2.0,-3.52,-0.465,9.348
5.47,2.0,-9.012,-1.227,4.158
1.59,1.6,-5.38,-0.736,-8.397
5.49,2.0,-5.038,-0.701,-8.61
4.3,2.0,-9.509,-1.34,-2.789
3.84,2.0,-3.037,-0.433,-9.518
6.29,2.0,-9.737,-1.414,1.787
3.85,2.0,-3.418,-0.503,9.384
6.28,2.0,-9.018,-1.33,4.112
4.24,2.0,-7.425,-1.101,6.607
5.48,2.0,-9.757,-1.449,-1.642
2.65,0.893,-9.075,-1.372,-3.971
4.95,2.0,-3.377,-0.516,9.398
4.8,2.0,-9.124,-1.398,3.848
6.57,2.0,-9.379,-1.448,3.153
5.03,2.0,-9.379,-1.449,3.153
5.88,2.0,-9.82,-1.59,-1.016
5.7,2.0,-9.434,-1.535,2.939
2.69,2.0,-3.515,-0.575,-9.344
3.85,2.0,-6.531,-1.084,-7.494
6.35,2.0,-7.453,-1.273,6.544
5.49,2.0,-9.196,-1.585,3.594
4.66,2.0,-9.758,-1.688,-1.391
4.63,2.0,-7.546,-1.326,-6.426
2.2,2.0,-6.458,-1.183,-7.543
6.22,2.0,-9.674,-1.774,1.81
2.74,2.0,-9.832,-1.807,-0.253
4.88,2.0,-9.677,-1.788,1.777
4.91,2.0,-4.963,-0.919,-8.633
5.57,2.0,-9.764,-1.807,1.185
4.66,2.0,-6.472,-1.217,-7.526
5.95,2.0,-7.592,-1.51,6.331
5.22,2.0,-9.715,-1.96,1.335
4.69,2.0,-4.755,-0.959,-8.745
3.04,2.0,-3.653,-0.748,-9.279
5.65,2.0,-9.66,-1.982,1.657
5.12,2.0,-9.387,-1.937,2.853
6.11,2.0,-9.576,-2.002,2.072
5.43,2.0,-3.856,-0.812,9.191
1.25,-0.238,-4.938,-1.043,-8.633
6.42,2.0,-9.764,-2.069,0.623
6.47,2.0,-9.507,-2.031,2.343
6.25,2.0,-6.481,-1.398,7.486
5.71,2.0,-9.478,-2.054,2.44
5.76,2.0,-8.662,-1.892,4.626
4.9,2.0,-8.089,-1.819,-5.592
6.02,2.0,-9.734,-2.23,0.533
4.93,2.0,-8.642,-1.983,4.624
6.32,2.0,-9.312,-2.159,2.936
4.33,2.0,-6.392,-1.509,-7.541
6.11,2.0,-9.713,-2.296,-0.62
4.89,2.0,-9.07,-2.148,3.624
4.25,2.0,-7.434,-1.765,-6.452
5.89,2.0,-4.807,-1.15,-8.693
6.25,2.0,-9.498,-2.273,2.151
1.76,-0.022,-5.443,-1.307,8.286
4.77,2.0,-9.586,-2.317,-1.657
4.03,2.0,-5.267,-1.279,-8.404
5.08,2.0,-5.268,-1.28,-8.403
4.62,2.0,-4.983,-1.211,-8.585
5.45,2.0,-0.826,-0.202,-9.964
5.23,2.0,-4.036,-0.996,9.095
3.39,2.0,-9.69,-2.398,0.593
5.61,2.0,-7.613,-1.898,6.2
2.89,2.0,-7.612,-1.899,6.2
5.17,2.0,-6.073,-1.544,-7.793
4.76,2.0,-9.228,-2.426,2.992
5.79,2.0,-9.642,-2.568,-0.665
5.37,2.0,-3.837,-1.027,9.177
4.88,2.0,-8.296,-2.233,5.118
5.99,2.0,-9.636,-2.609,-0.588
5.97,2.0,-9.218,-2.52,2.944
2.85,2.0,-9.459,-2.631,1.901
3.61,2.0,-3.049,-0.849,-9.486
4.83,2.0,-6.243,-1.777,-7.607
6.62,2.0,-9.594,-2.747,-0.639
5.2,2.0,-7.779,-2.295,5.849
4.71,2.0,-6.356,-1.891,-7.485
6.0,2.0,-8.938,-2.663,3.609
5.53,2.0,-8.847,-2.636,3.846
4.27,2.0,-6.168,-1.854,-7.65
4.8,2.0,-8.482,-2.56,4.637
5.15,2.0,-9.397,-2.868,-1.864
5.44,2.0,-3.995,-1.224,-9.086
5.57,2.0,-9.439,-2.91,-1.562
4.94,2.0,-8.783,-2.729,-3.926
6.25,2.0,-7.463,-2.342,6.23
5.95,2.0,-9.386,-2.948,-1.793
4.38,2.0,-9.493,-2.991,-0.965
4.32,2.0,-9.094,-2.867,3.012
5.91,2.0,-7.463,-2.355,6.225
4.23,2.0,-8.408,-2.727,4.676
5.04,2.0,-9.132,-2.97,-2.79
6.26,2.0,-8.983,-2.982,-3.227
5.31,2.0,-8.913,-2.99,-3.409
4.79,2.0,-3.562,-1.214,-9.265
5.77,2.0,-7.157,-2.448,6.541
5.21,2.0,-8.889,-3.06,-3.411
5.19,2.0,-9.317,-3.243,1.637
5.1,2.0,-8.051,-2.807,-5.226
4.72,2.0,-7.165,-2.521,6.504
4.78,2.0,-9.389,-3.307,0.953
5.14,2.0,-6.097,-2.166,7.625
4.74,2.0,-8.943,-3.185,-3.141
2.99,2.0,-8.654,-3.104,-3.935
5.6,2.0,-7.179,-2.624,6.448
2.75,2.0,-7.526,-2.762,-5.978
5.89,2.0,-9.326,-3.495,0.898
4.52,2.0,-4.538,-1.711,-8.745
5.36,2.0,-8.907,-3.375,-3.046
5.88,2.0,-9.312,-3.543,-0.858
2.23,0.057,-5.365,-2.058,8.184
4.52,2.0,-4.014,-1.541,-9.029
5.76,2.0,-9.289,-3.592,-0.9
5.04,2.0,-2.429,-0.946,-9.654
0.98,-0.235,-9.141,-3.564,-1.936
5.27,2.0,-9.065,-3.604,-2.2
6.62,2.0,-2.451,-0.982,-9.645
4.76,2.0,-8.922,-3.58,-2.752
4.97,2.0,-8.998,-3.655,2.382
5.65,2.0,-9.087,-3.728,1.877
5.04,2.0,-5.799,-2.385,-7.79
6.1,2.0,-9.173,-3.82,-1.127
3.9,2.0,-7.125,-2.989,-6.348
4.68,2.0,-9.151,-3.883,-1.09
6.01,2.0,-8.717,-3.702,-3.211
5.52,2.0,-8.862,-3.801,-2.649
5.21,2.0,-9.044,-3.884,-1.765
4.92,2.0,-9.15,-3.985,0.638
4.68,2.0,-6.009,-2.627,7.549
3.38,2.0,-9.158,-4.015,-0.104
5.7,2.0,-9.103,-4.03,-0.94
4.82,2.0,-7.342,-3.325,5.919
2.29,-0.171,-5.396,-2.513,-8.035
5.73,2.0,-8.507,-3.998,3.413
5.56,2.0,-0.665,-0.313,-9.973
5.63,2.0,-8.356,-3.943,3.826
5.03,2.0,-8.929,-4.241,-1.513
4.99,2.0,-5.236,-2.491,-8.147
5.35,2.0,-8.989,-4.339,0.617
5.55,2.0,-8.623,-4.229,-2.786
6.18,2.0,-8.62,-4.278,-2.717
4.23,2.0,-7.507,-3.73,-5.453
5.5,2.0,-8.741,-4.355,-2.152
4.64,2.0,-5.571,-2.798,-7.819
5.97,2.0,-8.051,-4.046,4.337
5.15,2.0,-7.202,-3.628,-5.913
4.5,2.0,-8.514,-4.304,3.0
5.41,2.0,-8.492,-4.3,-3.067
1.85,-0.099,-5.815,-2.948,7.583
4.19,2.0,-7.324,-3.79,-5.657
4.05,2.0,-8.545,-4.424,2.722
3.41,2.0,-6.631,-3.434,-6.651
3.47,2.0,-6.548,-3.395,-6.753
4.92,2.0,-8.272,-4.293,3.627
4.96,2.0,-8.432,-4.383,-3.112
4.58,2.0,-3.775,-1.995,9.043
4.32,2.0,-7.408,-3.932,-5.446
4.75,2.0,-7.473,-4.024,-5.288
5.71,2.0,-8.377,-4.511,3.079
5.53,2.0,-7.148,-3.863,-5.83
2.68,2.0,-8.325,-4.553,3.156
5.16,2.0,-8.77,-4.797,-0.262
2.55,-0.176,-5.939,-3.277,-7.348
5.9,2.0,-8.735,-4.865,0.183
5.02,2.0,-7.748,-4.32,4.616
3.83,2.0,-6.453,-3.661,-6.704
5.2,2.0,-7.88,-4.482,-4.222
5.76,2.0,-8.074,-4.598,3.697
3.87,2.0,-6.165,-3.512,-7.047
5.77,2.0,-7.848,-4.531,-4.228
6.23,2.0,-7.667,-4.479,4.6
4.23,2.0,-8.621,-5.06,0.27
4.34,2.0,-6.032,-3.544,-7.145
0.61,-0.231,-4.239,-2.543,-8.693
3.67,2.0,-3.703,-2.234,9.016
5.69,2.0,-1.951,-1.188,-9.736
4.36,2.0,-6.417,-3.934,-6.584
3.25,2.0,-7.611,-4.681,-4.49
2.06,1.011,-6.853,-4.228,-5.93
5.46,2.0,-8.398,-5.182,-1.618
6.43,2.0,-2.259,-1.397,9.641
5.26,2.0,-5.508,-3.451,7.599
4.8,2.0,-1.824,-1.149,9.765
6.45,2.0,-8.32,-5.25,-1.794
4.82,2.0,-7.63,-4.879,4.241
5.07,2.0,-7.439,-4.866,-4.58
4.18,2.0,-8.231,-5.391,-1.784
6.62,2.0,-5.166,-3.402,7.857
4.53,2.0,-5.166,-3.402,7.857
5.53,2.0,-8.124,-5.382,2.243
5.29,2.0,-8.189,-5.464,1.754
-0.05,1.239,-7.837,-5.27,3.287
4.07,2.0,-8.245,-5.562,-1.045
4.75,2.0,-5.173,-3.495,7.812
4.18,2.0,-5.743,-3.888,7.204
4.8,2.0,-6.709,-4.611,5.808
4.89,2.0,-1.287,-0.887,-9.877
4.52,2.0,-7.991,-5.549,-2.313
5.41,2.0,-7.999,-5.563,2.25
3.55,2.0,-5.695,-3.965,-7.2
5.14,2.0,-8.197,-5.715,-0.395
4.84,2.0,-7.868,-5.497,2.808
4.3,2.0,-4.53,-3.182,-8.328
4.05,2.0,-6.454,-4.543,-6.141
5.06,2.0,-1.397,-1.0,-9.851
4.41,2.0,-6.261,-4.508,-6.362
4.78,2.0,-7.18,-5.173,-4.657
6.22,2.0,-7.936,-5.736,-2.03
4.04,2.0,-4.978,-3.657,7.864
4.56,2.0,-5.66,-4.193,-7.098
4.33,2.0,-5.643,-4.183,-7.118
5.4,2.0,-7.579,-5.632,3.293
4.31,2.0,-0.884,-0.66,-9.939
6.18,2.0,-7.956,-5.963,-1.066
4.25,2.0,-1.976,-1.483,9.69
4.97,2.0,-6.948,-5.244,-4.923
4.81,2.0,-7.975,-6.021,-0.389
5.58,2.0,-5.139,-3.895,7.643
5.42,2.0,-7.91,-5.999,-1.201
11.01,2.0,-3.643,-2.791,-8.885
3.57,2.0,-6.803,-5.307,5.056
3.04,2.0,-6.182,-4.833,6.199
5.91,2.0,-7.279,-5.715,3.788
4.44,2.0,-5.006,-3.933,-7.711
4.47,2.0,-6.779,-5.425,4.961
2.33,-0.157,-5.771,-4.653,-6.712
4.05,2.0,-5.021,-4.135,-7.596
5.39,2.0,-5.496,-4.564,6.997
1.35,2.0,-3.737,-3.126,-8.733
-0.01,0.71,-3.737,-3.127,-8.732
4.49,2.0,-7.329,-6.189,2.826
3.78,2.0,-7.41,-6.281,2.373
4.86,2.0,-7.537,-6.417,1.42
5.55,2.0,-7.455,-6.352,2.021
2.3,-0.154,-5.15,-4.395,-7.36
3.18,2.0,-3.211,-2.754,-9.061
3.87,2.0,-7.537,-6.498,-0.986
5.7,2.0,-6.86,-5.923,-4.226
4.8,2.0,-6.767,-5.853,4.466
4.06,2.0,-6.177,-5.354,-5.761
4.92,2.0,-6.145,-5.388,-5.763
2.35,2.0,-6.695,-5.871,4.551
4.6,2.0,-7.185,-6.314,2.918
5.68,2.0,-7.503,-6.609,0.125
6.33,2.0,-7.22,-6.385,-2.666
5.15,2.0,-6.763,-5.984,-4.296
3.73,2.0,-7.478,-6.631,0.33
5.22,2.0,-4.553,-4.065,-7.921
5.61,2.0,-6.717,-6.015,-4.325
5.23,2.0,-6.681,-6.002,-4.397
3.83,2.0,-1.413,-1.271,-9.818
5.76,2.0,-6.643,-5.979,-4.485
5.76,2.0,-5.125,-4.668,7.207
5.32,2.0,-7.169,-6.53,-2.444
5.68,2.0,-4.87,-4.45,7.515
4.42,2.0,-6.505,-5.976,-4.689
5.15,2.0,-7.068,-6.516,-2.756
2.07,1.465,-2.007,-1.851,9.62
2.75,2.0,-7.061,-6.52,-2.763
4.93,2.0,-7.337,-6.783,-0.401
4.54,2.0,-6.928,-6.426,3.272
4.32,2.0,-5.306,-4.933,-6.893
5.27,2.0,-6.584,-6.266,-4.169
5.78,2.0,-7.087,-6.747,-2.062
6.09,2.0,-2.943,-2.81,-9.135
5.38,2.0,-4.343,-4.204,-7.966
5.08,2.0,-3.28,-3.188,-8.893
5.48,2.0,-7.028,-6.833,-1.978
4.47,2.0,-7.137,-6.964,-0.758
6.61,2.0,-6.991,-6.884,-1.935
2.68,2.0,-5.193,-5.127,-6.837
5.65,2.0,-6.3,-6.226,-4.642
5.88,2.0,-6.971,-6.904,-1.933
3.13,2.0,-5.265,-5.227,-6.705
5.64,2.0,-5.484,-5.465,6.329
4.91,2.0,-6.963,-7.023,-1.481
5.65,2.0,-0.827,-0.841,-9.93
3.49,2.0,-5.34,-5.431,6.48
5.83,2.0,-6.182,-6.297,-4.704
4.8,2.0,-6.349,-6.467,4.227
4.39,2.0,-6.976,-7.155,0.365
4.83,2.0,-4.684,-4.841,7.391
3.25,2.0,-6.279,-6.506,-4.271
4.52,2.0,-6.18,-6.424,4.532
5.65,2.0,-0.839,-0.875,-9.926
5.16,2.0,-3.031,-3.16,-8.99
3.91,2.0,-4.709,-4.924,-7.32
5.59,2.0,-4.604,-4.828,7.449
5.19,2.0,-6.589,-6.982,-2.799
4.93,2.0,-6.208,-6.616,4.206
5.67,2.0,-6.103,-6.567,4.431
4.07,2.0,-4.78,-5.164,-7.106
5.88,2.0,-0.61,-0.673,-9.959
3.88,2.0,-4.414,-4.9,-7.517
5.7,2.0,-4.413,-4.9,-7.518
4.54,2.0,-6.289,-6.999,-3.386
3.41,2.0,-4.105,-4.57,-7.891
4.83,2.0,-4.753,-5.318,-7.009
6.07,2.0,-6.261,-7.035,-3.362
6.47,2.0,-6.005,-6.757,-4.275
5.28,2.0,-5.772,-6.552,4.873
6.18,2.0,-6.293,-7.148,-3.052
4.91,2.0,-5.631,-6.4,-5.228
5.32,2.0,-6.563,-7.496,0.861
3.46,2.0,-5.496,-6.295,5.492
5.62,2.0,-6.566,-7.542,0.065
5.04,2.0,-3.17,-3.677,-8.743
2.61,-0.071,-6.44,-7.475,-1.63
5.02,2.0,-2.035,-2.364,9.501
4.07,2.0,-3.373,-3.932,-8.554
4.85,2.0,-2.892,-3.375,-8.958
4.35,2.0,-5.621,-6.571,-5.022
4.27,2.0,-4.344,-5.111,-7.417
2.87,2.0,-2.35,-2.774,-9.316
5.04,2.0,-6.448,-7.637,0.308
5.51,2.0,-5.584,-6.663,4.942
3.0,2.0,-1.996,-2.395,9.502
6.16,2.0,-6.079,-7.303,-3.116
6.14,2.0,-6.16,-7.408,-2.681
5.35,2.0,-6.393,-7.689,0.125
3.22,2.0,-4.842,-5.841,-6.514
3.57,2.0,-5.134,-6.218,-5.915
5.38,2.0,-5.344,-6.472,5.437
5.65,2.0,-4.234,-5.128,-7.468
4.99,2.0,-4.259,-5.173,-7.423
6.29,2.0,-6.196,-7.542,2.176
3.37,2.0,-4.506,-5.499,-7.033
4.54,2.0,-5.058,-6.2,-5.998
4.99,2.0,-5.457,-6.692,5.044
4.48,2.0,-3.222,-3.957,-8.6
6.11,2.0,-6.301,-7.763,-0.178
4.92,2.0,-6.185,-7.651,-1.792
4.31,2.0,-4.988,-6.186,6.071
6.51,2.0,-4.989,-6.189,6.066
5.36,2.0,-4.822,-5.994,-6.389
3.29,2.0,-3.228,-4.02,8.569
4.6,2.0,-4.874,-6.091,-6.257
5.16,2.0,-6.008,-7.539,2.66
3.66,2.0,-5.384,-6.881,4.864
5.64,2.0,-5.888,-7.554,-2.876
5.15,2.0,-6.131,-7.893,0.321
6.69,2.0,-5.822,-7.558,-2.997
5.82,2.0,-5.811,-7.62,-2.858
5.04,2.0,-4.582,-6.021,6.539
5.0,2.0,-1.322,-1.746,9.757
5.4,2.0,-1.725,-2.279,-9.583
4.98,2.0,-4.555,-6.032,6.547
6.22,2.0,-5.778,-7.679,2.766
5.53,2.0,-5.729,-7.666,-2.899
4.14,2.0,-5.111,-6.841,5.204
5.5,2.0,-5.983,-8.01,-0.207
4.61,2.0,-5.851,-7.919,-1.747
5.13,2.0,-5.23,-7.109,-4.702
2.22,0.032,-5.292,-7.196,4.496
3.8,2.0,-5.82,-7.923,1.829
2.8,2.0,-4.448,-6.074,-6.583
5.14,2.0,-4.588,-6.271,6.295
3.91,2.0,-5.699,-7.81,-2.553
6.1,2.0,-5.616,-7.698,3.033
4.55,2.0,-4.162,-5.723,-7.066
6.51,2.0,-5.659,-7.822,2.605
5.93,2.0,-5.63,-7.784,2.776
5.26,2.0,-5.771,-7.979,1.738
6.51,2.0,-5.858,-8.104,-0.098
4.11,2.0,-2.351,-3.257,-9.158
3.6,2.0,-5.151,-7.157,-4.716
5.25,2.0,-4.43,-6.201,6.475
4.34,2.0,-4.275,-5.997,-6.765
3.66,2.0,-5.02,-7.081,-4.966
5.36,2.0,-5.45,-7.705,-3.305
4.64,2.0,-4.62,-6.561,5.967
4.66,2.0,-4.739,-6.753,-5.651
4.97,2.0,-5.238,-7.5,-4.038
5.64,2.0,-1.631,-2.336,-9.586
6.0,2.0,-5.479,-7.897,2.761
4.64,2.0,-4.049,-5.848,-7.029
4.51,2.0,-5.349,-7.75,3.366
5.34,2.0,-5.53,-8.03,2.224
5.8,2.0,-5.375,-7.815,3.167
4.75,2.0,-5.335,-7.759,-3.367
5.23,2.0,-4.48,-6.557,-6.077
4.75,2.0,-4.636,-6.789,-5.694
3.81,2.0,-5.054,-7.405,4.43
5.57,2.0,-0.542,-0.798,-9.953
5.57,2.0,-4.716,-6.991,5.375
5.86,2.0,-5.585,-8.283,0.439
4.29,2.0,-1.182,-1.753,9.774
5.41,2.0,-5.381,-7.984,-2.701
2.63,1.167,-5.547,-8.245,1.119
6.15,2.0,-5.316,-7.933,2.968
5.39,2.0,-5.513,-8.337,-0.315
3.65,2.0,-5.314,-8.043,2.659
4.42,2.0,-5.458,-8.281,1.28
5.71,2.0,-5.307,-8.117,2.439
4.09,2.0,-5.15,-7.986,3.114
4.59,2.0,-4.84,-7.567,4.394
3.54,2.0,-5.377,-8.41,-0.598
5.21,2.0,-5.358,-8.435,0.383
3.71,2.0,-5.327,-8.427,0.781
3.97,2.0,-4.444,-7.041,-5.538
4.63,2.0,-4.807,-7.617,-4.345
4.79,2.0,-4.329,-6.876,5.829
5.09,2.0,-5.319,-8.451,-0.539
4.74,2.0,-4.973,-7.902,3.58
4.6,2.0,-3.892,-6.269,6.749
6.07,2.0,-5.116,-8.283,2.283
5.04,2.0,-4.928,-7.989,-3.448
4.59,2.0,-4.736,-7.699,-4.278
4.13,2.0,-5.01,-8.162,-2.879
5.35,2.0,-3.796,-6.232,6.838
5.87,2.0,-4.704,-7.726,-4.265
6.28,2.0,-5.142,-8.446,1.492
5.95,2.0,-4.893,-8.065,-3.319
2.83,2.0,-2.318,-3.825,-8.944
5.11,2.0,-1.886,-3.123,-9.311
5.63,2.0,-4.635,-7.677,-4.425
5.73,2.0,-3.807,-6.305,6.764
5.43,2.0,-4.068,-6.756,6.149
3.85,2.0,-4.943,-8.263,2.7
3.87,2.0,-4.466,-7.499,-4.881
5.14,2.0,-4.244,-7.126,-5.587
5.59,2.0,-4.244,-7.126,-5.587
5.54,2.0,-4.943,-8.329,2.489
4.14,2.0,-4.541,-7.677,4.521
4.95,2.0,-4.912,-8.354,-2.466
2.89,2.0,-4.528,-7.754,-4.402
3.42,2.0,-3.915,-6.789,-6.211
5.47,2.0,-4.781,-8.309,-2.846
2.29,-0.117,-4.604,-8.001,-3.846
5.53,2.0,-4.917,-8.584,-1.463
5.39,2.0,-4.146,-7.257,5.491
5.1,2.0,-4.716,-8.27,3.06
4.98,2.0,-4.289,-7.538,4.977
4.01,2.0,-2.57,-4.538,8.532
4.82,2.0,-4.529,-8.029,3.876
4.72,2.0,-3.397,-6.054,7.198
4.65,2.0,-3.186,-5.701,-7.573
4.63,2.0,-2.595,-4.659,-8.46
6.07,2.0,-4.839,-8.708,0.869
4.16,2.0,-4.739,-8.582,-1.972
2.56,-0.065,-4.51,-8.257,-3.388
4.9,2.0,-4.51,-8.258,-3.388
4.73,2.0,-3.351,-6.203,-7.092
4.22,2.0,-3.802,-7.047,-5.991
3.93,2.0,-4.435,-8.238,-3.53
4.31,2.0,-4.408,-8.239,-3.562
5.75,2.0,-4.594,-8.604,-2.206
5.63,2.0,-4.639,-8.691,1.718
5.0,2.0,-4.486,-8.443,2.932
6.25,2.0,-4.485,-8.443,2.933
5.69,2.0,-4.625,-8.741,1.484
4.23,2.0,-3.302,-6.262,7.063
6.13,2.0,-4.456,-8.45,2.958
4.73,2.0,-3.744,-7.114,5.947
5.57,2.0,-2.466,-4.702,-8.474
6.61,2.0,-3.32,-6.406,6.924
5.48,2.0,-1.119,-2.168,9.698
6.09,2.0,-4.369,-8.526,2.868
5.74,2.0,-4.177,-8.165,3.987
4.0,2.0,-4.281,-8.401,-3.332
4.93,2.0,-4.47,-8.773,-1.748
5.43,2.0,-4.485,-8.813,-1.486
5.67,2.0,-3.984,-7.841,-4.759
4.58,2.0,-4.001,-7.878,-4.683
6.36,2.0,-1.154,-2.277,9.669
5.46,2.0,-4.474,-8.901,0.875
5.78,2.0,-2.538,-5.057,-8.245
4.95,2.0,-2.595,-5.174,-8.154
5.24,2.0,-4.373,-8.756,-2.051
2.73,2.0,-4.439,-8.937,-0.644
5.23,2.0,-3.683,-7.443,5.571
5.13,2.0,-2.989,-6.077,-7.358
3.86,2.0,-1.953,-3.98,-8.964
5.72,2.0,-4.169,-8.499,3.224
5.49,2.0,-4.352,-8.885,-1.455
5.8,2.0,-3.803,-7.862,4.871
4.97,2.0,-2.788,-5.782,-7.668
4.95,2.0,-1.064,-2.219,9.693
4.8,2.0,-3.769,-7.928,-4.789
3.23,2.0,-4.278,-9.002,-0.818
5.44,2.0,-3.128,-6.654,-6.778
3.91,2.0,-2.926,-6.257,7.231
4.01,2.0,-2.712,-5.805,-7.678
4.68,2.0,-0.826,-1.778,-9.806
5.27,2.0,-0.827,-1.783,-9.805
4.55,2.0,-3.833,-8.279,-4.094
2.9,2.0,-3.769,-8.194,-4.32
3.74,2.0,-3.92,-8.595,3.281
4.82,2.0,-4.143,-9.099,0.18
4.86,2.0,-3.555,-7.81,5.134
5.2,2.0,-3.434,-7.567,5.563
5.4,2.0,-3.434,-7.577,5.549
2.73,2.0,-1.94,-4.357,8.789
4.48,2.0,-3.817,-8.584,-3.426
5.83,2.0,-4.03,-9.072,1.21
5.53,2.0,-3.187,-7.278,6.073
4.57,2.0,-3.891,-8.887,2.425
4.57,2.0,-3.673,-8.407,-3.979
4.22,2.0,-3.743,-8.716,-3.166
4.46,2.0,-2.659,-6.203,-7.379
4.62,2.0,-3.873,-9.104,-1.456
5.28,2.0,-1.71,-4.027,-8.992
4.94,2.0,-1.415,-3.333,9.321
4.9,2.0,-1.325,-3.138,-9.402
4.83,2.0,-2.89,-6.862,6.676
1.06,1.865,-3.448,-8.264,-4.451
6.44,2.0,-2.51,-6.067,7.543
4.79,2.0,-3.457,-8.368,-4.244
2.78,2.0,-3.553,-8.6,3.663
5.24,2.0,-3.564,-8.664,3.499
3.82,2.0,-3.788,-9.248,0.346
4.29,2.0,-3.623,-8.871,-2.859
4.45,2.0,-3.481,-8.631,-3.66
5.63,2.0,-3.705,-9.238,0.962
4.84,2.0,-3.647,-9.096,1.992
3.86,2.0,-0.71,-1.79,-9.813
4.86,2.0,-2.632,-6.689,-6.952
4.2,2.0,-2.702,-6.868,6.748
5.5,2.0,-1.491,-3.871,-9.099
2.82,2.0,-3.162,-8.225,-4.728
5.53,2.0,-2.157,-5.633,7.976
5.07,2.0,-2.155,-5.63,7.978
5.77,2.0,-3.566,-9.334,-0.405
2.54,0.038,-3.476,-9.195,-1.834
4.86,2.0,-2.281,-6.161,7.539
6.92,2.0,-3.387,-9.38,0.734
5.77,2.0,-3.385,-9.381,0.736
4.84,2.0,-1.451,-4.038,9.033
2.81,2.0,-2.868,-8.02,5.24
5.89,2.0,-1.244,-3.483,-9.291
5.92,2.0,-2.991,-8.4,4.527
5.74,2.0,-3.349,-9.42,0.206
3.48,2.0,-2.568,-7.344,6.283
4.23,2.0,-0.712,-2.041,-9.764
6.02,2.0,-3.193,-9.475,0.178
5.15,2.0,-3.144,-9.375,1.492
4.21,2.0,-0.44,-1.314,9.904
6.72,2.0,-2.831,-8.568,-4.31
6.07,2.0,-3.123,-9.493,0.36
5.22,2.0,-3.087,-9.468,0.914
1.91,1.447,-1.096,-3.407,-9.338
4.82,2.0,-2.112,-6.62,7.191
3.77,2.0,-1.551,-4.905,-8.575
4.64,2.0,-2.961,-9.367,-1.871
2.29,1.144,-2.478,-7.881,-5.634
5.48,2.0,-2.97,-9.465,1.262
5.73,2.0,-2.586,-8.283,4.971
5.51,2.0,-2.947,-9.553,0.212
5.03,2.0,-2.666,-8.688,4.172
3.0,2.0,-2.307,-7.53,-6.163
6.51,2.0,-2.821,-9.239,2.584
3.56,2.0,-2.292,-7.537,-6.159
5.34,2.0,-2.453,-8.147,5.255
4.7,2.0,-2.099,-7.085,-6.738
4.39,2.0,-2.795,-9.438,1.765
3.62,2.0,-2.081,-7.09,-6.738
5.23,2.0,-2.799,-9.54,-1.072
6.06,2.0,-2.519,-8.649,4.341
5.35,2.0,-2.64,-9.112,3.162
4.88,2.0,-1.159,-4.042,9.073
6.4,2.0,-1.156,-4.059,9.066
5.57,2.0,-2.504,-8.847,-3.931
5.48,2.0,-2.263,-8.05,-5.484
6.27,2.0,-2.433,-8.703,4.282
3.19,2.0,-2.65,-9.504,1.629
3.12,2.0,-1.48,-5.394,-8.289
4.06,2.0,-1.562,-5.789,-8.003
5.74,2.0,-2.34,-8.757,-4.224
3.92,2.0,-2.21,-8.289,5.139
4.82,2.0,-2.537,-9.645,-0.736
5.27,2.0,-2.1,-8.063,5.529
6.28,2.0,-2.375,-9.159,-3.237
5.27,2.0,-1.47,-5.802,-8.011
6.19,2.0,-1.989,-7.903,5.795
4.83,2.0,-1.974,-8.04,-5.61
4.91,2.0,-1.373,-5.647,8.138
4.89,2.0,-2.303,-9.478,2.205
5.41,2.0,-1.82,-7.89,5.869
3.17,2.0,-0.911,-4.011,9.115
2.43,2.0,-2.068,-9.401,-2.71
6.2,2.0,-1.933,-8.911,4.105
3.32,2.0,-1.51,-7.127,-6.85
5.32,2.0,-2.024,-9.619,1.837
2.78,2.0,-1.904,-9.497,2.485
3.12,2.0,-1.769,-8.901,4.201
3.16,2.0,-1.56,-7.853,5.992
4.33,2.0,-1.731,-8.772,-4.478
4.72,2.0,-1.882,-9.821,-0.078
6.31,2.0,-0.031,-0.165,9.999
4.8,2.0,-1.551,-8.232,5.461
6.81,2.0,-1.642,-8.787,-4.482
4.64,2.0,-1.461,-7.82,6.059
6.59,2.0,-1.661,-8.963,-4.113
5.14,2.0,-1.661,-8.962,-4.113
5.03,2.0,-1.764,-9.661,1.885
5.51,2.0,-1.191,-6.813,7.223
5.38,2.0,-1.441,-8.313,5.369
4.32,2.0,-1.658,-9.608,-2.223
5.13,2.0,-1.545,-8.968,4.147
4.39,2.0,-1.58,-9.194,-3.602
4.76,2.0,-0.625,-3.731,-9.257
3.27,2.0,-1.495,-8.939,-4.226
5.39,2.0,-0.56,-3.354,-9.404
5.21,2.0,-1.079,-6.673,-7.369
5.3,2.0,-1.404,-8.705,-4.717
4.15,2.0,-1.258,-7.871,6.038
5.7,2.0,-1.436,-9.095,3.901
2.84,2.0,-0.854,-5.595,-8.244
3.31,2.0,-0.833,-5.474,-8.327
5.19,2.0,-0.938,-6.273,-7.731
4.16,2.0,-1.334,-9.025,-4.095
4.34,2.0,-1.452,-9.868,0.722
5.83,2.0,-0.963,-6.588,7.462
4.28,2.0,-1.231,-8.584,-4.98
2.79,2.0,-0.787,-6.064,7.912
4.41,2.0,-1.143,-8.906,4.401
2.7,2.0,-1.012,-7.891,-6.059
5.78,2.0,-0.067,-0.525,9.986
3.6,2.0,-0.616,-4.857,-8.719
4.78,2.0,-1.137,-9.067,-4.061
5.66,2.0,-1.079,-8.729,4.757
2.84,2.0,-0.79,-6.396,-7.647
5.07,2.0,-0.454,-3.696,9.281
4.89,2.0,-0.692,-5.667,8.21
4.35,2.0,-0.072,-0.591,9.982
4.86,2.0,-0.689,-5.669,8.209
6.61,2.0,-0.323,-2.699,9.623
1.62,-0.231,-0.916,-7.923,-6.033
5.8,2.0,-1.09,-9.8,1.665
2.08,0.155,-1.065,-9.702,2.175
5.23,2.0,-0.513,-4.686,8.819
6.47,2.0,-0.997,-9.215,-3.753
4.56,2.0,-0.73,-6.844,-7.254
5.35,2.0,-0.673,-6.581,7.499
4.77,2.0,-0.364,-3.605,9.321
1.86,0.406,-0.723,-7.278,-6.82
5.76,2.0,-0.893,-9.07,4.117
3.54,2.0,-0.941,-9.595,-2.655
4.58,2.0,-0.956,-9.854,-1.412
5.25,2.0,-0.554,-5.781,-8.141
3.82,2.0,-0.622,-6.918,7.194
4.76,2.0,-0.556,-6.482,-7.594
4.24,2.0,-0.79,-9.717,-2.228
4.57,2.0,-0.241,-3.056,9.519
5.81,2.0,-0.241,-3.055,9.519
5.56,2.0,-0.695,-9.068,4.157
2.39,-0.171,-0.593,-7.746,-6.297
5.73,2.0,-0.661,-9.088,4.12
4.86,2.0,-0.671,-9.268,-3.695
2.76,2.0,-0.718,-9.942,0.796
5.12,2.0,-0.427,-6.165,-7.862
6.16,2.0,-0.672,-9.967,0.45
3.61,2.0,-0.266,-4.262,-9.043
3.42,2.0,-0.523,-8.837,4.652
4.53,2.0,-0.48,-8.83,-4.669
2.99,2.0,-0.414,-7.635,-6.445
3.75,2.0,-0.527,-9.975,0.472
5.09,2.0,-0.44,-9.006,4.324
5.02,2.0,-0.301,-6.316,7.747
5.02,2.0,-0.104,-2.254,9.742
6.78,2.0,-0.288,-6.634,7.477
4.78,2.0,-0.328,-7.643,-6.44
5.17,2.0,-0.224,-7.656,6.429
3.73,2.0,-0.154,-5.463,8.375
6.18,2.0,-0.202,-9.069,-4.208
5.43,2.0,-0.065,-3.089,9.511
5.47,2.0,-0.18,-8.982,4.392
3.86,2.0,-0.13,-7.959,6.053
2.24,1.521,-0.092,-6.226,7.825
3.7,2.0,-0.085,-8.725,4.886
4.41,2.0,-0.056,-8.644,5.029
3.32,2.0,-0.042,-9.855,-1.698
4.74,2.0,-0.008,-9.148,-4.038
6.11,2.0,0.0,-1.736,9.848
4.67,2.0,0.002,-9.576,2.882
5.74,2.0,0.001,-1.736,9.848
4.79,2.0,0.011,-9.971,0.762
4.62,2.0,0.021,-9.979,-0.644
3.93,2.0,0.028,-9.987,0.511
6.27,2.0,0.058,-9.555,-2.95
4.26,2.0,0.061,-9.298,3.681
4.42,2.0,0.076,-9.997,0.228
5.25,2.0,0.097,-9.346,3.557
6.22,2.0,0.101,-9.21,3.895
5.37,2.0,0.113,-9.115,-4.112
4.77,2.0,0.133,-9.897,-1.423
5.89,2.0,0.154,-9.108,-4.125
4.66,2.0,0.19,-8.695,-4.936
4.03,2.0,0.238,-9.988,0.437
2.98,2.0,0.219,-8.62,-5.064
4.96,2.0,0.244,-9.254,3.781
3.65,2.0,0.186,-6.413,-7.671
5.05,2.0,0.264,-8.607,5.085
4.64,2.0,0.315,-9.879,1.518
3.71,2.0,0.316,-9.856,1.661
3.84,2.0,0.288,-8.761,4.812
5.79,2.0,0.307,-8.975,4.399
5.83,2.0,0.307,-8.975,4.4
4.33,2.0,0.166,-4.433,-8.962
4.37,2.0,0.357,-9.341,3.553
5.1,2.0,0.364,-9.387,3.428
5.71,2.0,0.416,-9.967,0.696
5.47,2.0,0.214,-4.689,-8.83
4.52,2.0,0.341,-6.944,-7.188
4.96,2.0,0.443,-8.524,5.211
3.84,2.0,0.56,-9.315,-3.593
4.99,2.0,0.262,-4.313,9.018
5.49,2.0,0.578,-9.272,-3.7
5.29,2.0,0.62,-9.332,-3.539
5.96,2.0,0.622,-9.353,-3.484
5.96,2.0,0.241,-3.615,9.32
6.79,2.0,0.268,-3.616,9.32
3.1,2.0,0.616,-7.988,-5.985
5.3,2.0,0.761,-9.072,4.138
4.33,2.0,0.7,-8.053,5.887
4.92,2.0,0.82,-9.238,3.74
4.22,2.0,0.289,-3.187,9.474
4.85,2.0,0.908,-9.941,0.589
5.61,2.0,0.792,-8.636,4.979
2.72,2.0,0.794,-8.639,-4.974
5.12,2.0,0.802,-8.72,4.828
3.55,2.0,0.272,-2.956,9.549
3.23,2.0,0.928,-9.944,-0.505
4.35,2.0,0.483,-4.748,-8.788
5.47,2.0,0.265,-2.567,-9.661
4.66,2.0,1.018,-9.826,-1.553
3.85,2.0,0.958,-9.237,3.709
4.98,2.0,0.539,-5.152,8.554
1.79,-0.031,0.869,-8.207,-5.647
5.11,2.0,0.814,-7.672,6.362
5.58,2.0,0.936,-8.542,-5.114
4.81,2.0,1.034,-9.307,-3.509
4.82,2.0,0.468,-4.11,9.104
3.49,2.0,0.816,-6.902,-7.19
5.2,2.0,1.184,-9.93,0.034
2.82,2.0,1.1,-8.965,-4.293
4.1,2.0,0.822,-6.5,-7.555
4.67,2.0,1.23,-9.6,-2.515
5.38,2.0,1.291,-9.91,-0.346
4.63,2.0,0.635,-4.608,-8.852
4.92,2.0,0.961,-6.891,-7.183
5.93,2.0,1.389,-9.901,-0.175
5.07,2.0,0.972,-6.909,-7.164
4.77,2.0,0.771,-5.385,8.391
5.67,2.0,1.132,-7.719,-6.256
6.31,2.0,1.133,-7.719,-6.255
4.62,2.0,1.077,-7.316,-6.732
5.49,2.0,1.346,-9.033,-4.073
6.53,2.0,1.369,-9.016,-4.103
3.85,2.0,1.514,-9.78,-1.434
0.03,-0.001,1.251,-7.694,6.264
5.76,2.0,1.633,-9.866,-0.054
6.22,2.0,1.661,-8.995,-4.041
4.7,2.0,1.811,-9.708,-1.573
5.03,2.0,1.047,-5.561,8.245
4.01,2.0,0.595,-3.129,-9.479
4.88,2.0,1.868,-9.718,-1.439
5.11,2.0,1.49,-7.703,-6.201
4.67,2.0,1.48,-7.554,6.384
4.59,2.0,1.482,-7.56,6.376
4.34,2.0,1.538,-7.772,6.102
5.73,2.0,1.539,-7.773,6.101
5.02,2.0,1.942,-9.803,0.359
3.17,2.0,1.763,-8.735,-4.538
4.19,2.0,1.853,-9.179,3.51
5.37,2.0,1.857,-9.058,-3.809
5.37,2.0,0.505,-2.464,9.679
5.89,2.0,2.014,-9.794,-0.168
4.34,2.0,1.936,-9.301,3.12
4.22,2.0,2.037,-9.755,-0.828
5.2,2.0,1.575,-7.45,-6.482
5.71,2.0,0.887,-4.119,-9.069
5.46,2.0,1.53,-7.068,-6.906
5.6,2.0,1.559,-7.092,-6.875
5.22,2.0,2.016,-9.158,-3.473
5.93,2.0,1.811,-8.207,5.419
5.22,2.0,1.82,-8.23,5.381
3.52,2.0,1.811,-8.153,5.499
6.29,2.0,2.038,-9.034,-3.772
4.63,2.0,1.128,-4.966,8.606
6.08,2.0,2.219,-9.734,-0.579
4.22,2.0,1.054,-4.545,-8.845
5.43,2.0,2.105,-9.068,3.653
5.18,2.0,1.399,-5.98,-7.892
5.58,2.0,1.856,-7.771,6.014
5.68,2.0,2.174,-9.056,-3.642
4.86,2.0,2.16,-8.966,-3.866
4.82,2.0,0.754,-3.117,9.472
4.22,2.0,1.884,-7.772,6.004
4.57,2.0,2.184,-8.967,3.85
5.29,2.0,0.099,-0.406,-9.991
5.0,2.0,2.198,-8.962,-3.854
2.05,-0.134,2.141,-8.706,-4.43
4.08,2.0,1.722,-6.991,6.94
4.62,2.0,2.422,-9.675,0.733
4.98,2.0,2.423,-9.674,0.733
5.67,2.0,1.306,-5.164,8.463
4.4,2.0,0.952,-3.751,-9.221
4.83,2.0,2.451,-9.641,-1.019
5.56,2.0,2.471,-9.68,0.442
5.02,2.0,2.317,-9.066,-3.528
3.52,2.0,2.325,-9.035,-3.601
4.85,2.0,1.521,-5.832,-7.98
5.14,2.0,1.257,-4.808,-8.678
4.83,2.0,2.021,-7.715,-6.033
5.91,2.0,2.462,-9.389,2.403
3.25,2.0,2.141,-8.139,5.401
5.27,2.0,2.478,-9.397,2.355
4.02,2.0,2.484,-9.331,2.6
4.94,2.0,2.192,-8.178,5.321
5.51,2.0,1.477,-5.444,8.257
5.0,2.0,1.809,-6.584,7.306
4.02,2.0,2.646,-9.592,-1.0
2.6,0.062,2.339,-8.349,-4.982
5.4,2.0,2.705,-9.606,-0.645
4.74,2.0,2.018,-7.141,-6.704
3.76,2.0,2.587,-8.921,-3.704
5.4,2.0,1.667,-5.725,8.028
5.4,2.0,2.79,-9.577,-0.703
2.99,2.0,2.733,-9.316,2.396
3.43,2.0,2.84,-9.55,-0.851
5.17,2.0,1.744,-5.856,-7.917
4.23,2.0,2.28,-7.647,-6.027
3.32,2.0,2.55,-8.481,-4.644
5.07,2.0,2.827,-9.398,1.92
5.25,2.0,2.339,-7.734,5.892
5.2,2.0,2.446,-8.071,5.373
4.57,2.0,2.234,-7.269,-6.494
5.23,2.0,2.949,-9.497,1.058
5.11,2.0,0.691,-2.219,9.726
4.11,2.0,2.355,-7.531,-6.143
6.26,2.0,1.22,-3.884,9.134
2.88,2.0,2.798,-8.905,-3.588
4.1,2.0,2.327,-7.376,-6.339
5.13,2.0,1.682,-5.202,8.373
5.93,2.0,2.633,-8.13,5.193
3.07,2.0,1.183,-3.612,9.25
5.35,2.0,3.088,-9.41,-1.381
5.14,2.0,3.159,-9.48,0.4
4.43,2.0,2.453,-7.357,6.313
5.0,2.0,1.693,-5.067,8.453
5.65,2.0,3.007,-8.823,3.621
4.86,2.0,2.927,-8.557,-4.267
4.45,2.0,0.927,-2.71,9.581
4.76,2.0,3.04,-8.801,3.647
4.35,2.0,2.573,-7.433,6.175
6.25,2.0,1.164,-3.357,-9.348
5.59,2.0,3.265,-9.414,0.843
3.8,2.0,1.97,-5.632,8.025
4.88,2.0,3.143,-8.92,-3.248
5.46,2.0,3.062,-8.679,3.911
5.28,2.0,3.263,-9.237,2.01
5.1,2.0,3.36,-9.417,0.189
6.4,2.0,3.373,-9.414,0.059
5.53,2.0,3.327,-9.184,2.143
6.03,2.0,3.346,-9.209,2.0
4.98,2.0,3.427,-9.347,-0.944
5.46,2.0,3.444,-9.387,-0.156
4.6,2.0,1.418,-3.861,9.115
3.92,2.0,3.321,-8.921,-3.065
4.52,2.0,3.356,-9.01,-2.749
5.84,2.0,3.319,-8.895,-3.141
3.96,2.0,2.518,-6.679,-7.004
5.22,2.0,3.172,-8.388,4.425
5.03,2.0,2.058,-5.442,-8.133
4.27,2.0,2.52,-6.633,-7.046
3.96,2.0,2.717,-7.088,-6.51
4.99,2.0,3.12,-8.114,4.943
6.27,2.0,3.443,-8.925,2.913
5.17,2.0,3.544,-9.119,2.069
5.02,2.0,3.308,-8.476,-4.148
5.14,2.0,3.428,-8.762,3.387
3.36,2.0,3.639,-9.298,0.543
5.45,2.0,3.33,-8.51,-4.061
5.17,2.0,2.958,-7.495,5.923
5.6,2.0,3.45,-8.734,3.436
5.57,2.0,3.415,-8.635,-3.71
4.64,2.0,3.686,-9.296,0.059
5.73,2.0,2.276,-5.673,7.914
4.44,2.0,3.43,-8.415,4.173
5.82,2.0,3.436,-8.405,4.19
5.79,2.0,3.785,-9.25,0.34
6.34,2.0,3.565,-8.676,3.466
3.76,2.0,2.363,-5.725,7.851
6.29,2.0,2.202,-5.279,-8.203
5.03,2.0,3.849,-9.217,-0.487
3.05,2.0,3.406,-8.15,4.689
5.12,2.0,3.407,-8.149,4.689
4.74,2.0,3.214,-7.594,5.657
4.67,2.0,1.363,-3.197,9.377
4.45,2.0,3.958,-9.093,1.284
5.0,2.0,3.774,-8.62,3.383
5.39,2.0,3.501,-7.972,4.919
5.12,2.0,3.964,-8.996,-1.833
4.88,2.0,2.695,-6.11,-7.443
6.03,2.0,3.244,-7.304,6.01
5.64,2.0,3.696,-8.298,-4.182
4.49,2.0,2.614,-5.84,7.685
4.59,2.0,3.715,-8.276,-4.208
4.36,2.0,4.095,-9.12,-0.225
4.93,2.0,4.072,-9.051,-1.223
5.67,2.0,3.95,-8.739,2.834
5.45,2.0,4.125,-9.074,-0.81
5.18,2.0,4.176,-9.038,0.941
4.68,2.0,3.633,-7.847,5.023
5.41,2.0,3.084,-6.655,6.797
6.33,2.0,3.871,-8.319,-3.976
4.39,2.0,4.023,-8.617,3.092
5.64,2.0,4.254,-9.049,-0.108
5.3,2.0,4.084,-8.686,-2.806
4.39,2.0,4.071,-8.626,3.003
5.99,2.0,2.732,-5.74,7.719
6.33,2.0,4.216,-8.818,2.112
5.06,2.0,4.155,-8.661,-2.777
5.28,2.0,4.236,-8.824,2.05
5.5,2.0,3.938,-8.099,4.348
4.89,2.0,3.493,-7.14,6.067
6.25,2.0,4.288,-8.736,2.301
2.86,2.0,3.12,-6.328,7.087
5.89,2.0,4.41,-8.877,1.325
2.72,2.0,4.396,-8.791,1.842
4.87,2.0,4.213,-8.416,-3.381
5.0,2.0,3.725,-7.436,5.553
3.68,2.0,4.282,-8.459,3.179
5.33,2.0,2.515,-4.935,-8.326
3.84,2.0,1.535,-3.007,9.413
5.75,2.0,4.47,-8.708,2.048
5.01,2.0,4.325,-8.399,3.279
7.91,2.0,3.895,-7.437,5.434
5.18,2.0,3.62,-6.911,6.255
5.03,2.0,2.794,-5.332,7.985
0.76,0.221,4.592,-8.749,1.541
5.38,2.0,4.566,-8.699,-1.868
5.12,2.0,4.58,-8.704,1.808
4.9,2.0,4.301,-8.168,3.845
5.88,2.0,4.445,-8.342,-3.263
6.24,2.0,4.461,-8.358,3.201
3.87,2.0,4.712,-8.818,0.176
5.14,2.0,2.547,-4.727,8.436
4.57,2.0,4.337,-8.034,4.08
5.76,2.0,4.723,-8.687,-1.491
4.71,2.0,4.729,-8.687,1.471
5.7,2.0,4.746,-8.685,-1.431
6.49,2.0,4.746,-8.684,-1.433
5.6,2.0,4.8,-8.772,0.048
4.12,2.0,3.589,-6.525,-6.674
3.71,2.0,4.792,-8.706,1.116
4.91,2.0,2.947,-5.336,7.927
4.7,2.0,4.341,-7.844,-4.431
4.95,2.0,3.791,-6.848,6.223
5.71,2.0,4.646,-8.38,2.863
5.28,2.0,4.761,-8.568,1.981
3.89,2.0,3.977,-7.152,5.748
4.54,2.0,4.345,-7.763,-4.566
5.54,2.0,4.706,-8.338,2.888
5.01,2.0,4.744,-8.388,-2.671
3.51,2.0,4.669,-8.189,3.337
4.84,2.0,4.451,-7.791,-4.414
5.68,2.0,4.57,-7.982,3.924
6.2,2.0,1.165,-2.026,9.723
4.37,2.0,4.074,-7.075,-5.775
5.3,2.0,4.106,-7.123,-5.692
5.15,2.0,3.989,-6.914,6.024
5.33,2.0,4.77,-8.258,3.01
5.75,2.0,1.963,-3.388,-9.202
3.97,2.0,1.476,-2.541,-9.558
4.66,2.0,4.462,-7.643,4.657
5.06,2.0,3.24,-5.535,7.672
5.22,2.0,2.151,-3.671,9.05
5.32,2.0,1.986,-3.376,-9.201
5.69,2.0,4.931,-8.374,-2.358
5.23,2.0,4.603,-7.812,4.216
6.27,2.0,2.18,-3.688,9.036
4.43,2.0,4.515,-7.615,-4.65
4.51,2.0,1.923,-3.239,9.264
5.8,2.0,4.927,-8.192,2.936
5.51,2.0,5.114,-8.5,1.267
5.67,2.0,5.164,-8.562,-0.124
5.09,2.0,4.881,-8.031,3.419
6.53,2.0,5.077,-8.331,-2.193
5.4,2.0,2.445,-4.008,8.829
5.38,2.0,4.24,-6.894,5.874
5.08,2.0,4.818,-7.794,4.006
4.93,2.0,3.184,-5.126,-7.974
5.97,2.0,5.3,-8.479,-0.118
3.55,2.0,2.151,-3.419,-9.148
4.38,2.0,1.135,-1.801,9.771
4.93,2.0,4.283,-6.761,5.996
6.51,2.0,5.017,-7.879,3.57
5.51,2.0,4.81,-7.509,4.525
3.24,2.0,5.42,-8.402,-0.143
5.7,2.0,2.543,-3.932,8.836
5.51,2.0,4.854,-7.49,4.51
6.34,2.0,5.318,-8.192,-2.146
5.91,2.0,4.875,-7.507,4.459
5.84,2.0,5.33,-8.174,-2.184
5.44,2.0,5.491,-8.356,-0.176
4.8,2.0,3.76,-5.718,7.292
4.28,2.0,3.029,-4.602,8.345
3.8,2.0,3.773,-5.721,7.283
5.19,2.0,4.849,-7.31,4.801
4.94,2.0,5.336,-8.041,2.621
4.93,2.0,4.435,-6.666,5.991
3.96,2.0,3.75,-5.587,7.398
5.18,2.0,5.111,-7.613,3.989
4.5,2.0,4.938,-7.338,4.666
6.11,2.0,5.21,-7.701,3.681
6.3,2.0,5.476,-8.09,-2.137
5.3,2.0,5.107,-7.516,4.174
4.3,2.0,5.517,-8.054,-2.166
4.77,2.0,4.455,-6.495,6.161
5.86,2.0,5.259,-7.651,-3.715
3.58,2.0,5.531,-8.043,-2.172
5.58,2.0,4.537,-6.572,6.018
5.14,2.0,4.66,-6.739,5.733
5.28,2.0,5.399,-7.754,-3.275
5.71,2.0,2.664,-3.818,8.85
4.77,2.0,5.617,-7.973,-2.209
6.09,2.0,5.573,-7.901,-2.552
3.05,2.0,5.581,-7.896,-2.551
5.5,2.0,5.288,-7.41,4.138
2.23,0.673,4.438,-6.209,6.462
5.6,2.0,4.324,-6.037,-6.698
4.43,2.0,4.97,-6.85,5.327
5.64,2.0,4.336,-5.974,-6.746
1.94,-0.118,3.256,-4.414,-8.361
5.73,2.0,3.896,-5.216,7.591
5.08,2.0,5.694,-7.603,-3.125
5.63,2.0,4.702,-6.264,6.217
5.38,2.0,0.899,-1.19,9.888
6.12,2.0,6.022,-7.962,-0.586
4.77,2.0,5.758,-7.582,-3.059
5.9,2.0,4.878,-6.395,5.942
4.01,2.0,5.234,-6.859,5.056
5.96,2.0,0.94,-1.231,9.879
4.21,2.0,2.758,-3.608,8.91
4.91,2.0,6.068,-7.933,-0.503
6.74,2.0,5.766,-7.523,-3.187
5.94,2.0,5.767,-7.522,-3.187
4.94,2.0,3.999,-5.209,7.542
6.03,2.0,5.988,-7.783,1.89
6.21,2.0,4.893,-6.32,6.009
5.44,2.0,4.006,-5.159,7.572
5.18,2.0,1.594,-2.049,9.657
4.03,2.0,6.078,-7.695,1.96
4.61,2.0,5.081,-6.393,5.772
5.12,2.0,4.437,-5.582,-7.011
5.39,2.0,6.063,-7.626,2.254
4.64,2.0,6.065,-7.536,2.533
4.75,2.0,3.084,-3.823,-8.711
6.4,2.0,5.666,-6.988,4.365
4.91,2.0,6.312,-7.744,-0.445
5.59,2.0,5.667,-6.931,4.456
6.32,2.0,5.406,-6.585,5.236
3.64,2.0,6.141,-7.479,2.52
3.11,2.0,4.304,-5.242,-7.348
4.86,2.0,3.026,-3.684,-8.791
5.42,2.0,6.23,-7.57,1.973
4.31,2.0,6.371,-7.705,-0.193
4.81,2.0,5.947,-7.18,3.616
5.06,2.0,5.822,-7.029,4.086
5.69,2.0,6.214,-7.489,2.303
5.07,2.0,6.3,-7.566,1.751
5.24,2.0,6.187,-7.421,-2.581
5.15,2.0,6.408,-7.677,0.085
3.77,2.0,6.17,-7.377,2.742
5.11,2.0,3.161,-3.766,-8.708
5.15,2.0,6.11,-7.278,-3.113
5.53,2.0,5.462,-6.45,5.345
6.01,2.0,6.262,-7.379,2.518
1.25,0.092,4.556,-5.362,7.106
6.51,2.0,1.64,-1.925,-9.675
5.14,2.0,2.562,-3.001,-9.189
5.99,2.0,1.552,-1.816,-9.711
5.41,2.0,4.149,-4.849,7.698
5.75,2.0,0.847,-0.986,9.915
5.59,2.0,2.589,-3.0,9.182
4.43,2.0,6.318,-7.302,2.601
4.51,2.0,4.047,-4.654,-7.872
4.92,2.0,5.959,-6.802,4.269
3.42,2.0,2.66,-3.034,-9.15
3.41,2.0,3.116,-3.545,8.816
4.22,2.0,5.687,-6.447,5.108
4.13,2.0,5.995,-6.77,-4.269
5.91,2.0,6.17,-6.963,-3.667
2.48,1.021,5.501,-6.206,5.588
5.15,2.0,6.386,-7.177,2.777
4.27,2.0,6.386,-7.177,2.777
4.53,2.0,5.364,-5.988,5.947
3.78,2.0,6.589,-7.339,-1.65
4.43,2.0,6.657,-7.41,-0.876
5.57,2.0,6.648,-7.396,1.047
5.11,2.0,4.826,-5.337,-6.945
4.81,2.0,4.66,-5.133,7.207
5.41,2.0,2.438,-2.676,-9.322
4.9,2.0,4.662,-5.111,-7.221
6.01,2.0,6.583,-7.207,2.172
6.32,2.0,6.686,-7.309,1.368
4.89,2.0,5.615,-6.129,-5.56
5.06,2.0,4.857,-5.297,6.954
5.99,2.0,6.769,-7.295,-0.98
5.06,2.0,4.226,-4.551,-7.838
4.12,2.0,6.076,-6.526,-4.527
6.06,2.0,5.708,-6.122,-5.472
4.56,2.0,6.075,-6.507,4.555
5.55,2.0,6.793,-7.275,-0.96
4.73,2.0,6.757,-7.205,-1.561
4.8,2.0,4.902,-5.2,6.995
5.03,2.0,6.09,-6.386,4.704
5.78,2.0,6.573,-6.879,-3.077
3.67,2.0,3.615,-3.782,-8.522
5.19,2.0,6.736,-7.0,2.372
5.54,2.0,6.769,-7.032,2.176
5.49,2.0,6.875,-7.064,-1.684
3.94,2.0,5.257,-5.388,6.583
5.3,2.0,6.492,-6.59,3.799
5.51,2.0,6.897,-6.992,1.881
5.3,2.0,7.023,-7.08,0.749
6.26,2.0,6.673,-6.696,-3.261
4.74,2.0,4.772,-4.779,7.375
6.21,2.0,7.064,-7.03,-0.825
5.38,2.0,4.923,-4.873,7.212
4.67,2.0,6.013,-5.946,-5.337
5.32,2.0,5.595,-5.452,-6.243
5.53,2.0,7.159,-6.908,-1.015
4.82,2.0,6.777,-6.522,-3.396
5.63,2.0,7.178,-6.897,0.959
5.13,2.0,1.62,-1.555,-9.745
3.72,2.0,5.201,-4.982,6.938
5.17,2.0,4.176,-3.989,-8.164
5.94,2.0,7.198,-6.864,1.038
4.08,2.0,6.927,-6.576,-2.963
5.69,2.0,6.274,-5.953,-5.019
5.55,2.0,5.452,-5.154,-6.611
4.56,2.0,4.899,-4.624,7.39
5.2,2.0,5.678,-5.348,6.258
6.05,2.0,5.679,-5.348,6.257
4.49,2.0,6.604,-6.206,-4.227
5.3,2.0,6.834,-6.342,-3.615
5.45,2.0,0.134,-0.124,-9.998
6.25,2.0,6.891,-6.339,-3.511
4.5,2.0,7.216,-6.636,-1.972
4.7,2.0,7.268,-6.64,1.759
6.07,2.0,7.275,-6.636,1.745
3.21,2.0,6.445,-5.755,5.034
5.06,2.0,2.54,-2.26,-9.404
4.47,2.0,7.389,-6.51,1.738
3.74,2.0,5.916,-5.198,6.163
5.17,2.0,7.052,-6.15,-3.527
5.31,2.0,7.277,-6.34,-2.617
3.92,2.0,7.51,-6.539,0.915
4.22,2.0,5.864,-5.034,6.347
4.41,2.0,6.235,-5.329,5.721
4.71,2.0,6.435,-5.499,-5.325
5.4,2.0,7.231,-6.179,-3.088
5.83,2.0,7.586,-6.468,-0.788
7.07,2.0,7.261,-6.187,-3.001
5.04,2.0,5.484,-4.665,6.94
2.45,0.257,3.509,-2.981,8.877
5.19,2.0,3.245,-2.738,9.054
4.39,2.0,4.56,-3.831,-8.033
4.8,2.0,5.814,-4.846,-6.535
5.87,2.0,7.666,-6.372,-0.795
5.81,2.0,7.627,-6.339,1.28
4.08,2.0,7.262,-5.982,3.388
4.28,2.0,7.392,-6.08,-2.896
5.16,2.0,7.686,-6.286,1.186
5.99,2.0,7.64,-6.246,-1.619
5.38,2.0,7.267,-5.876,-3.559
5.48,2.0,7.581,-6.129,-2.229
5.76,2.0,5.873,-4.738,-6.561
6.38,2.0,7.781,-6.253,-0.593
5.71,2.0,7.692,-6.162,-1.693
5.48,2.0,7.792,-6.238,-0.62
5.93,2.0,6.273,-4.999,5.972
6.1,2.0,4.531,-3.596,-8.157
4.21,2.0,3.267,-2.588,-9.09
3.77,2.0,7.252,-5.734,-3.813
5.78,2.0,7.328,-5.764,-3.616
5.3,2.0,6.27,-4.927,6.034
5.39,2.0,6.975,-5.466,4.634
5.42,2.0,3.101,-2.428,9.192
3.23,2.0,2.628,-2.041,9.43
4.5,2.0,7.334,-5.693,-3.715
6.52,2.0,6.762,-5.233,-5.186
5.22,2.0,5.447,-4.2,7.259
4.52,2.0,7.267,-5.578,4.01
2.9,2.0,7.937,-6.005,-0.971
5.29,2.0,6.013,-4.527,-6.584
5.97,2.0,6.631,-4.985,-5.584
3.98,2.0,5.625,-4.163,7.144
4.87,2.0,6.304,-4.632,6.23
5.7,2.0,7.57,-5.558,-3.434
5.73,2.0,7.265,-5.269,-4.411
6.1,2.0,6.799,-4.903,-5.453
5.04,2.0,6.178,-4.45,6.483
4.51,2.0,7.654,-5.506,-3.332
6.19,2.0,8.08,-5.778,1.153
4.68,2.0,8.059,-5.761,-1.367
5.46,2.0,7.677,-5.488,3.308
4.76,2.0,3.811,-2.72,8.836
5.66,2.0,8.113,-5.759,1.006
5.1,2.0,8.174,-5.747,0.392
3.69,2.0,7.85,-5.492,-2.867
5.09,2.0,5.967,-4.171,6.855
3.73,2.0,1.796,-1.241,-9.759
5.16,2.0,7.984,-5.51,-2.427
6.07,2.0,6.23,-4.299,6.535
4.55,2.0,2.64,-1.816,9.473
5.24,2.0,7.572,-5.203,-3.949
4.69,2.0,5.167,-3.548,7.792
5.66,2.0,8.243,-5.657,0.224
5.3,2.0,8.207,-5.627,0.99
5.73,2.0,6.22,-4.259,6.571
4.72,2.0,7.814,-5.337,-3.234
5.88,2.0,8.008,-5.449,-2.487
5.69,2.0,6.497,-4.406,6.196
4.23,2.0,4.291,-2.908,8.552
5.96,2.0,8.018,-5.407,-2.546
4.49,2.0,7.272,-4.898,4.809
2.38,1.52,8.172,-5.502,1.715
4.34,2.0,7.925,-5.32,2.982
4.14,2.0,7.488,-5.02,4.328
4.35,2.0,6.97,-4.66,-5.45
5.1,2.0,8.211,-5.486,-1.579
4.25,2.0,4.021,-2.675,8.756
5.29,2.0,7.68,-5.08,3.899
6.0,2.0,8.236,-5.437,-1.612
5.57,2.0,8.188,-5.392,-1.971
4.23,2.0,5.449,-3.58,7.582
2.85,2.0,8.035,-5.266,-2.778
5.63,2.0,8.359,-5.468,0.469
5.53,2.0,4.099,-2.676,8.72
5.02,2.0,7.191,-4.682,-5.135
5.07,2.0,7.288,-4.65,5.026
5.34,2.0,8.056,-5.125,2.971
5.52,2.0,2.942,-1.86,-9.375
5.27,2.0,1.072,-0.677,-9.919
5.52,2.0,7.442,-4.628,4.817
5.09,2.0,7.649,-4.73,4.372
5.08,2.0,8.273,-5.105,-2.343
3.0,2.0,6.775,-4.155,-6.069
5.74,2.0,4.703,-2.857,8.35
6.17,2.0,4.558,-2.732,-8.471
5.54,2.0,8.403,-5.002,2.092
4.4,2.0,4.942,-2.913,-8.191
6.13,2.0,4.442,-2.603,-8.573
5.04,2.0,2.501,-1.455,9.572
6.0,2.0,8.604,-4.961,1.17
5.43,2.0,7.63,-4.368,-4.764
5.6,2.0,8.683,-4.959,0.106
5.61,2.0,8.457,-4.829,2.27
5.65,2.0,8.595,-4.905,1.436
5.55,2.0,4.613,-2.608,8.481
6.39,2.0,8.334,-4.694,-2.918
5.55,2.0,8.674,-4.844,-1.136
4.69,2.0,4.783,-2.671,-8.366
4.74,2.0,8.725,-4.871,-0.376
5.83,2.0,8.56,-4.778,1.974
4.26,2.0,3.746,-2.081,9.035
5.26,2.0,3.953,-2.194,8.92
6.45,2.0,7.588,-4.189,-4.987
5.29,2.0,8.762,-4.817,-0.158
5.27,2.0,4.01,-2.2,8.893
5.07,2.0,4.08,-2.235,8.852
5.69,2.0,7.681,-4.189,4.843
4.86,2.0,8.747,-4.765,0.882
2.95,2.0,8.784,-4.78,-0.056
5.62,2.0,4.441,-2.415,-8.628
4.47,2.0,6.779,-3.676,-6.367
4.29,2.0,8.541,-4.616,-2.397
3.77,2.0,7.961,-4.277,4.281
5.79,2.0,8.201,-4.368,3.698
1.73,-0.07,6.029,-3.198,-7.309
4.5,2.0,7.413,-3.925,-5.445
4.99,2.0,7.324,-3.876,-5.598
5.8,2.0,8.391,-4.416,-3.176
5.58,2.0,7.411,-3.89,5.472
4.79,2.0,2.69,-1.403,9.529
4.28,2.0,7.424,-3.865,5.472
4.94,2.0,7.479,-3.888,-5.38
3.52,2.0,8.822,-4.583,1.08
6.45,2.0,8.292,-4.293,3.58
5.43,2.0,8.702,-4.5,-2.005
3.39,2.0,4.683,-2.416,8.499
5.05,2.0,4.529,-2.32,8.609
6.04,2.0,8.646,-4.385,-2.452
5.33,2.0,8.356,-4.153,-3.596
5.45,2.0,7.924,-3.937,-4.659
4.18,2.0,4.879,-2.405,8.391
4.79,2.0,6.742,-3.302,-6.606
4.14,2.0,7.106,-3.467,6.122
5.11,2.0,6.725,-3.264,-6.643
5.34,2.0,8.778,-4.244,-2.221
4.17,2.0,8.921,-4.311,-1.354
5.75,2.0,8.969,-4.321,-0.939
5.49,2.0,1.951,-0.932,-9.763
5.75,2.0,4.127,-1.964,8.895
2.87,2.0,4.482,-2.126,-8.683
5.96,2.0,8.802,-4.151,-2.301
5.09,2.0,1.505,-0.702,-9.861
5.35,2.0,8.982,-4.179,-1.361
5.37,2.0,9.025,-4.187,1.009
4.55,2.0,6.247,-2.879,7.258
4.78,2.0,7.999,-3.674,4.746
4.82,2.0,8.885,-4.072,2.114
5.12,2.0,8.454,-3.871,-3.681
3.86,2.0,9.091,-4.159,-0.242
6.42,2.0,6.336,-2.863,-7.187
5.62,2.0,6.343,-2.853,-7.185
5.53,2.0,8.288,-3.711,-4.189
4.42,2.0,5.591,-2.502,7.905
6.2,2.0,8.532,-3.814,3.559
5.79,2.0,9.105,-4.048,-0.843
5.76,2.0,8.89,-3.937,-2.339
4.55,2.0,5.942,-2.629,7.601
5.28,2.0,2.788,-1.233,-9.524
4.8,2.0,9.155,-4.015,0.24
5.55,2.0,8.791,-3.797,-2.881
5.76,2.0,9.154,-3.951,0.766
5.83,2.0,1.786,-0.77,9.809
5.52,2.0,3.864,-1.659,9.073
4.51,2.0,3.89,-1.665,-9.061
4.78,2.0,9.172,-3.9,0.819
5.47,2.0,7.149,-3.011,-6.311
3.65,2.0,9.219,-3.874,-0.004
5.6,2.0,9.107,-3.813,1.587
4.07,2.0,4.832,-2.022,8.519
3.97,2.0,6.693,-2.797,-6.883
4.34,2.0,6.212,-2.588,7.397
4.12,2.0,6.671,-2.771,-6.915
5.45,2.0,1.79,-0.743,9.81
5.51,2.0,9.211,-3.817,0.773
5.64,2.0,7.786,-3.224,5.384
6.36,2.0,8.946,-3.692,-2.518
4.52,2.0,6.749,-2.779,6.836
4.82,2.0,9.089,-3.735,-1.853
6.43,2.0,8.312,-3.405,-4.395
3.76,2.0,5.917,-2.412,7.692
4.29,2.0,7.826,-3.182,-5.35
5.76,2.0,0.652,-0.264,-9.975
6.39,2.0,9.099,-3.691,-1.892
6.43,2.0,8.709,-3.493,3.458
4.91,2.0,4.363,-1.741,-8.828
5.88,2.0,9.301,-3.662,-0.275
5.21,2.0,8.713,-3.402,-3.536
4.04,2.0,9.326,-3.61,-0.02
5.08,2.0,2.628,-1.012,9.595
5.73,2.0,7.188,-2.764,6.379
6.28,2.0,7.096,-2.707,-6.505
5.85,2.0,7.101,-2.691,-6.507
4.64,2.0,5.819,-2.194,7.831
5.04,2.0,9.338,-3.502,-0.737
5.19,2.0,4.171,-1.546,8.956
5.84,2.0,8.841,-3.267,3.342
4.89,2.0,7.289,-2.68,6.3
6.33,2.0,8.845,-3.229,3.368
4.5,2.0,6.733,-2.434,6.981
4.18,2.0,8.378,-3.022,-4.547
3.41,2.0,9.251,-3.3,1.879
5.25,2.0,7.191,-2.565,6.458
4.8,2.0,8.217,-2.92,4.895
6.12,2.0,8.22,-2.896,-4.903
2.07,1.61,6.449,-2.263,-7.3
2.93,2.0,8.158,-2.849,5.033
6.4,2.0,9.375,-3.263,-1.212
4.84,2.0,7.086,-2.457,-6.615
4.68,2.0,8.944,-3.098,-3.228
5.11,2.0,7.047,-2.423,6.668
6.27,2.0,8.94,-3.014,3.316
4.84,2.0,5.638,-1.896,-8.039
4.13,2.0,1.421,-0.475,-9.887
3.97,2.0,8.699,-2.888,3.998
4.2,2.0,9.279,-3.074,2.109
5.24,2.0,8.953,-2.928,-3.357
5.68,2.0,9.222,-3.009,-2.429
6.19,2.0,9.356,-3.017,-1.832
3.49,2.0,5.949,-1.917,-7.806
4.05,2.0,9.265,-2.939,-2.35
3.5,2.0,3.847,-1.219,9.15
3.51,2.0,8.671,-2.734,4.163
5.91,2.0,7.096,-2.225,6.685
5.32,2.0,1.637,-0.513,-9.852
5.99,2.0,8.313,-2.567,-4.93
4.95,2.0,6.959,-2.126,6.86
5.16,2.0,9.427,-2.864,1.708
4.46,2.0,8.037,-2.437,-5.428
3.73,2.0,9.487,-2.873,-1.319
5.8,2.0,9.385,-2.803,-2.014
6.03,2.0,6.338,-1.889,-7.501
6.2,2.0,9.519,-2.794,-1.254
3.27,2.0,9.233,-2.706,-2.726
6.04,2.0,3.271,-0.959,-9.401
5.53,2.0,9.213,-2.696,-2.802
6.11,2.0,9.599,-2.798,0.186
4.91,2.0,9.49,-2.756,1.533
4.2,2.0,8.103,-2.326,-5.379
5.6,2.0,7.191,-2.049,6.64
5.72,2.0,6.442,-1.823,-7.428
5.45,2.0,9.004,-2.52,3.546
1.17,0.145,8.373,-2.336,-4.943
5.76,2.0,9.449,-2.567,2.033
5.43,2.0,9.652,-2.611,0.168
6.22,2.0,9.666,-2.562,0.032
4.11,2.0,5.852,-1.544,-7.96
6.23,2.0,9.601,-2.51,-1.229
3.62,2.0,7.157,-1.854,6.733
6.18,2.0,9.624,-2.465,-1.145
5.09,2.0,7.113,-1.82,6.789
5.12,2.0,7.968,-2.005,-5.7
2.44,1.655,8.559,-2.143,4.707
4.48,2.0,9.68,-2.419,0.666
4.64,2.0,6.231,-1.548,7.666
5.37,2.0,5.712,-1.407,-8.087
2.49,-0.002,9.371,-2.303,2.623
5.44,2.0,9.628,-2.349,-1.339
4.84,2.0,4.95,-1.174,8.609
4.48,2.0,8.907,-2.11,-4.026
4.28,2.0,7.057,-1.666,-6.886
5.62,2.0,7.575,-1.787,-6.279
4.54,2.0,9.603,-2.261,1.635
4.76,2.0,8.789,-2.065,4.3
5.3,2.0,6.719,-1.562,7.24
5.68,2.0,6.353,-1.474,7.581
4.41,2.0,2.458,-0.569,9.677
5.42,2.0,9.744,-2.219,0.371
3.68,2.0,9.099,-2.04,-3.612
5.05,2.0,9.647,-2.16,1.509
5.68,2.0,4.978,-1.11,8.601
4.71,2.0,9.022,-2.004,-3.82
5.39,2.0,9.62,-2.132,1.706
3.88,2.0,6.876,-1.513,-7.101
5.91,2.0,7.08,-1.555,6.889
5.15,2.0,9.666,-2.066,1.516
6.19,2.0,8.726,-1.862,4.516
4.53,2.0,6.368,-1.338,7.593
4.22,2.0,9.747,-1.969,-1.054
6.51,2.0,8.646,-1.689,4.733
4.24,2.0,9.692,-1.889,-1.579
4.93,2.0,9.734,-1.855,-1.344
3.7,2.0,9.81,-1.855,0.573
3.99,2.0,5.174,-0.972,-8.502
4.82,2.0,6.447,-1.202,7.549
4.41,2.0,9.706,-1.803,-1.596
5.54,2.0,7.441,-1.374,-6.537
5.98,2.0,7.335,-1.347,6.662
4.75,2.0,3.667,-0.669,9.279
4.41,2.0,8.295,-1.507,-5.378
4.99,2.0,9.702,-1.756,-1.67
5.2,2.0,9.571,-1.726,-2.327
5.56,2.0,9.804,-1.755,-0.893
5.44,2.0,6.507,-1.162,7.504
5.81,2.0,7.309,-1.293,6.701
5.05,2.0,9.807,-1.714,0.938
4.58,2.0,9.019,-1.564,4.026
5.58,2.0,8.498,-1.467,5.063
5.77,2.0,7.746,-1.335,6.182
5.35,2.0,8.381,-1.406,5.271
5.19,2.0,9.53,-1.567,-2.595
6.28,2.0,9.223,-1.515,3.556
3.96,2.0,9.269,-1.511,-3.437
5.09,2.0,9.643,-1.567,2.133
4.96,2.0,4.596,-0.711,8.853
5.56,2.0,8.346,-1.29,5.356
4.42,2.0,9.073,-1.381,3.972
4.38,2.0,9.256,-1.381,-3.525
5.53,2.0,5.993,-0.879,-7.957
4.95,2.0,9.894,-1.438,0.219
5.75,2.0,7.249,-1.047,6.809
6.26,2.0,9.896,-1.424,0.196
5.99,2.0,8.961,-1.272,4.253
4.27,2.0,9.841,-1.384,1.111
5.5,2.0,0.435,-0.061,-9.99
4.54,2.0,9.665,-1.309,2.209
5.22,2.0,7.685,-0.968,6.325
6.24,2.0,9.242,-1.147,-3.644
6.39,2.0,9.923,-1.22,-0.19
4.38,2.0,7.845,-0.93,-6.132
4.7,2.0,9.278,-1.087,-3.57
5.33,2.0,9.177,-1.067,3.827
4.97,2.0,8.487,-0.969,5.199
5.91,2.0,9.934,-1.125,-0.218
5.55,2.0,7.587,-0.843,6.459
5.63,2.0,8.288,-0.921,5.519
4.69,2.0,7.316,-0.799,-6.771
5.68,2.0,9.94,-1.028,0.367
3.81,2.0,6.856,-0.673,7.249
6.26,2.0,9.526,-0.932,2.895
5.49,2.0,9.445,-0.912,3.157
4.29,2.0,7.248,-0.694,6.854
5.35,2.0,6.338,-0.579,7.713
3.21,2.0,2.133,-0.193,9.768
6.07,2.0,6.839,-0.614,-7.27
4.97,2.0,9.656,-0.854,-2.457
4.13,2.0,9.914,-0.87,0.981
4.15,2.0,7.127,-0.611,6.988
5.3,2.0,8.443,-0.715,-5.31
5.36,2.0,9.478,-0.764,-3.095
4.82,2.0,9.49,-0.757,-3.06
4.49,2.0,9.965,-0.782,0.311
4.49,2.0,9.652,-0.729,-2.511
5.09,2.0,9.812,-0.713,1.793
4.93,2.0,8.694,-0.608,4.903
5.24,2.0,9.473,-0.654,-3.136
5.28,2.0,9.456,-0.578,-3.203
4.97,2.0,6.881,-0.42,7.244
4.95,2.0,9.964,-0.592,0.608
4.88,2.0,5.194,-0.294,8.54
5.18,2.0,6.388,-0.355,-7.686
5.49,2.0,9.975,-0.525,-0.482
5.43,2.0,4.656,-0.227,8.847
4.59,2.0,8.809,-0.426,-4.715
5.77,2.0,9.988,-0.46,0.188
5.95,2.0,8.751,-0.395,4.824
5.77,2.0,9.861,-0.372,1.618
5.17,2.0,9.454,-0.357,-3.241
5.59,2.0,9.981,-0.35,0.511
5.1,2.0,1.388,-0.048,-9.903
5.06,2.0,9.443,-0.31,3.276
5.3,2.0,9.813,-0.316,1.899
5.93,2.0,9.98,-0.308,-0.55
6.29,2.0,9.989,-0.302,0.365
4.51,2.0,5.371,-0.132,8.434
6.22,2.0,9.922,-0.211,1.231
5.72,2.0,1.362,-0.015,-9.907
5.0,2.0,4.337,-0.046,-9.011
4.63,2.0,9.052,-0.089,4.249
6.28,2.0,9.62,-0.069,-2.731
4.88,2.0,9.981,-0.058,-0.62
5.13,2.0,6.053,-0.028,-7.96
4.88,2.0,5.627,-0.024,8.266
4.03,2.0,9.928,-0.03,1.195
4.49,2.0,4.135,-0.002,-9.105
"""

dfl_str = """\
ID1,ID2
15,71
71,150
268,150
150,117
117,102
1472,1427
1932,2197
2197,2215
2628,2613
2613,2602
2613,2554
2554,2619
2672,2619
2554,2501
2501,2490
2554,2502
2869,2949
2949,2989
2989,3007
3007,3028
3028,3069
3069,3120
3120,3142
2949,2976
2976,2952
2976,3018
3018,3060
3060,3073
3073,3108
2771,2869
2373,2303
2303,2247
2247,2224
2224,2300
2300,2290
2290,2289
2289,2373
387,274
274,241
241,236
869,868
868,730
730,691
691,677
761,677
761,869
1913,1873
1873,1923
1923,1986
1986,1959
1959,1903
1903,1902
1902,1873
1873,1842
1842,1832
616,641
641,642
524,669
2694,2701
2701,2813
2813,2847
2847,2885
2885,2912
2847,2859
2859,2813
2701,2765
2813,2784
1331,1446
1446,1501
1501,1505
1505,1547
1547,1522
1522,1474
1474,1454
1454,1343
1248,1200
1200,933
1327,1343
1327,1248
933,960
1200,1165
240,185
185,116
116,74
74,17
1911,1855
1855,1814
1814,1844
1844,1850
1850,1834
1834,1833
1833,1804
1804,1787
1833,1860
1834,1907
1907,1955
1844,1705
1705,1674
1674,1627
1627,1585
1585,1587
2966,3061
3061,2864
2864,2840
2840,2966
3061,3173
3173,2864
290,328
222,83
83,35
83,148
148,182
182,231
231,323
323,356
356,370
370,336
336,222
309,356
309,355
355,367
367,419
419,413
413,374
374,328
328,344
344,367
1190,1489
1489,1646
1917,2011
1917,1991
1006,1027
1027,1012
1012,976
976,1026
1026,1033
1033,1053
1053,1074
1018,1023
1023,1033
956,953
956,928
956,976
1018,991
923,1018
1012,1006
1106,1083
1269,1256
1256,1194
1256,1261
1261,1187
1261,1299
926,913
913,860
860,842
842,866
842,812
812,776
1770,1772
1772,1672
2486,2504
2504,2512
2512,2515
2515,2509
2509,2497
2031,2020
2020,2035
2035,2064
2064,2077
2077,2105
2105,2114
1532,1550
1550,1568
1568,1561
1561,1532
1561,1567
1567,1590
1590,1615
1615,1599
1599,1568
1683,1671
1720,1639
1685,1679
1679,1641
1641,1631
1631,1628
1631,1692
1692,1679
1690,1745
2530,2565
2565,2600
2600,2703
2703,2752
2703,2767
2767,2826
2826,2901
2703,2634
2634,2568
2726,2736
2736,2747
2747,2769
2769,2759
2759,2736
786,821
786,624
624,570
2345,2350
2350,2296
2296,2307
2307,2345
2345,2518
2518,2607
2607,2403
2403,2260
2260,2174
2174,2115
2115,2017
2017,1856
1856,1688
1688,1577
2824,2828
2828,2848
2848,2832
2832,2824
206,245
245,300
300,324
324,358
358,360
360,406
406,453
522,574
574,595
595,626
626,514
514,479
479,452
452,421
421,376
376,400
400,441
441,478
478,496
496,629
629,651
651,667
667,711
711,716
716,634
383,437
959,1030
1030,1066
1066,1061
1061,977
1066,1100
1100,1119
1100,1122
1100,1080
1080,1038
1038,1099
1038,998
1038,972
972,943
972,930
930,906
906,877
3100,3010
3010,2955
2955,3045
3045,3113
3113,3100
3045,3086
3045,3059
2955,2951
2951,2923
2322,2349
2349,2287
2287,2273
2273,2267
2267,2214
2214,2199
2199,2163
2163,2086
2214,2210
2210,2190
2190,2169
2210,2250
2250,2297
2297,2266
2335,2351
2351,2377
2351,2335
2250,2267
562,359
359,423
1255,1236
1236,1233
1233,1271
1271,1273
1273,1255
1273,1287
1287,1334
1334,1374
1374,1362
1362,1359
1359,1417
1417,1442
1442,1471
1471,1517
1517,1582
1582,1611
1611,1766
1766,1785
45,515
515,357
357,311
311,252
2842,2737
2737,2792
2792,2842
2973,3017
3017,3011
3011,2997
2997,2993
2993,3020
3020,3011
1607,1553
1553,1439
1439,1435
1435,1459
1459,1552
1552,1607
1459,1453
1453,1423
1423,1406
1552,1553
884,859
859,826
826,784
784,720
784,844
844,820
820,769
769,701
784,769
720,741
720,721
701,720
721,719
741,742
2090,2039
2039,1989
1989,1940
1940,1964
1964,2039
1520,1475
1475,1436
1436,1379
1436,1520
2108,2081
2108,2037
2037,2002
2002,2003
2002,1952
2037,2049
2049,1978
1978,1916
1978,1908
1916,1892
1916,1952
1349,1345
1203,1081
1081,1015
1015,921
2432,2442
2442,2459
2459,2488
2488,2470
2470,2442
778,646
2835,2804
2804,2780
905,942
942,1042
1042,931
1042,1174
1174,1112
1715,1601
1601,1686
1686,1699
1699,1715
2149,2164
2164,2181
2181,2118
2118,2164
2118,2149
2887,3053
3053,1894
1894,2887
2311,2331
2261,2331
2311,2246
2246,2161
2161,2205
2205,2261
2261,2301
814,800
800,780
875,899
899,888
888,852
899,873
873,856
856,814
814,831
831,726
726,780
780,759
759,790
790,856
759,657
657,663
663,668
668,681
657,660
660,672
888,873
2707,2858
2858,2762
2762,2667
2667,2707
2667,2647
2647,2437
2437,2479
2479,2667
2479,2463
2463,2405
2405,2380
2380,2463
2380,2334
26,3096
3092,3046
3046,2959
3092,3062
3062,3054
3054,2953
2953,2904
3096,3055
3055,3041
3041,2963
2963,2902
15,3092
15,26
3092,3096
499,529
529,537
537,534
534,495
495,462
462,425
425,392
462,430
430,426
426,390
147,137
137,46
46,147
137,197
197,238
238,137
137,191
191,46
46,47
47,19
19,46
989,837
837,828
3082,3040
3040,2931
2931,2915
2915,2962
2962,3021
3021,3078
124,164
124,174
174,164
164,198
198,224
224,259
259,237
237,214
214,195
195,126
126,96
96,37
37,3213
3213,3176
3176,3181
3181,3149
3149,3122
3122,3153
3153,3176
1059,960
960,992
992,1089
1089,1165
1165,1170
1242,1257
1257,1276
564,571
571,536
536,498
498,564
3159,120
120,3129
3129,3159
2309,2328
2328,2337
2337,2317
2317,2263
2263,2235
2235,2231
2231,2226
2226,2201
2201,2187
2187,2110
2187,2107
2187,2122
2419,2431
2431,2451
2078,2080
2080,2070
2070,2036
2036,2073
2073,2100
2100,2076
2076,2073
2476,2404
2404,2324
2324,2319
2319,2279
1479,1438
2592,2605
2605,2589
2605,2638
2638,2660
2401,2417
2394,2410
2410,2371
2371,2336
2371,2401
2401,2410
2410,2495
2495,2445
2445,2401
2445,2417
2417,2387
2495,2505
2505,2474
2474,2445
2474,2483
2483,2498
2498,2531
2531,2541
2505,2578
2578,2655
2655,2642
2642,2627
2627,2548
2627,2547
761,643
643,610
628,806
580,590
580,539
539,464
628,610
628,611
611,580
610,601
601,590
590,520
2418,2415
2222,1995
1995,2095
2095,2222
299,277
277,234
234,299
2980,3123
3123,36
3123,56
1830,1792
1792,1736
1736,1640
1640,1540
1540,1537
1537,1612
1612,1640
1612,1603
1603,1549
1549,1455
1549,1464
1537,1420
1420,1375
1375,1309
1375,1300
1420,1414
1414,1218
1218,1368
1368,1540
335,2306
2306,2218
2218,2068
2068,2159
2159,1998
1998,1939
1939,2068
1177,1246
1246,1262
1262,1351
1351,1424
1424,1512
1512,1494
1494,1447
1447,1364
1364,1314
1314,1177
1602,1648
1648,1707
1707,1796
1796,1868
1868,1874
1874,1918
1796,1811
1811,1853
1853,1928
1811,1743
1743,1753
1743,1707
1113,1035
1035,1172
1172,1113
1172,1058
1172,1208
1208,1306
1306,1172
2561,2648
"""
